// Dynare shell which solves and estimates the RANK model
//
// Daeha Cho, December, 2022

//----------------------------------------------------------------
// Load parameters
//----------------------------------------------------------------

// Load in files containing parameters
economicParameters = load('economicParameters');
approximationParameters = load('approximationParameters');
steadyStateAggregates = load('steadyStateAggregates');
grids = load('grids');
polynomials = load('polynomials');

// Define economic parameters
parameters bbetaL ssigma aaBar bbetaH Hshare Hshare_T aalpha ddelta ss2prime rrhouk ppsi
	xxiP iiotaP ggamma rrhoxBar kkappa MMBar bb iiotaW xxiW FF rrhoR pphiPi pphiX
    rrhoetaP rrhomuu rrhoupsilon rrhoTFP rrhozeta rrhorhox rrhopiStar rrhog
    ssigmaetaP ssigmamuu ssigmaupsilon ssigmaTFP ssigmaR ssigmazeta ssigmarhox 
    ssigmapiStar ssigmag ssigmaW;


//Load in their values
@#define nEconomicParameters = 42
for iParam = 1 : @{nEconomicParameters}
	parameterName = deblank(M_.param_names(iParam,:));
	if isfield(economicParameters,parameterName)
		M_.params(iParam) = eval(['economicParameters.' parameterName]);
	end
end

// Define model option parameter
parameters modelOption_;

modelOption_ = modelOption; 

@#define nCounter = nEconomicParameters + 1

// Define steady state aggregate variables
parameters RSS PiSS muuSS nSS fSS wSS mcSS rkSS kSS lambdaSS uSS uaSS thetaSS vSS rhoxSS
           etaPSS pStarSS h1PSS h2PSS DeltaPSS xSS bgSS ttauSS gSS gtildeSS dSS ySS invSS cSS cHSS cLSS cH_TSS cL_TSS;


//Load in their values
@#define nSteadyStateAggregates = 33
for iParam = 1 : @{nSteadyStateAggregates}
	parameterName = deblank(M_.param_names(@{nCounter}+iParam,:));
	if isfield(steadyStateAggregates,parameterName)
		M_.params(@{nCounter}+iParam) = eval(['steadyStateAggregates.' parameterName]);
	end
end

@#define nCounter = nCounter + nSteadyStateAggregates



//----------------------------------------------------------------
// Define variables
//----------------------------------------------------------------

@#include "variablesBlock_RANK.mod"

//----------------------------------------------------------------
// Model equations
//----------------------------------------------------------------

model;

@#include "equationsBlock_RANK.mod"

end;

//----------------------------------------------------------------
// Specify shock process
//----------------------------------------------------------------

shocks;
    var epsetaP; stderr 1;
    var epsmuu; stderr 1;
    var epsupsilon; stderr 1;
    var epsTFP; stderr 1;
    var epsR; stderr 1;
    var epszeta; stderr 1;
    var epsrhox; stderr 1;
    var epsg; stderr 1;
    var epsW; stderr 1;
end;

options_.steadystate.nocheck = 1;
//resid(1);

@#if exer_option==1
//----------------------------------------------------------------
//   Bayesian estimation
//----------------------------------------------------------------

// Observables
varobs obsgC obsgI obsgG obsF obsS obsgWE obsPi obsFF;



// Estimated parameters
estimated_params;


ssigmaetaP, paramMean(1), inv_gamma_pdf, 0.15, 1;
ssigmamuu, paramMean(2), inv_gamma_pdf, 1, 1;
ssigmaupsilon, paramMean(3), inv_gamma_pdf, 0.5, 1;
ssigmaR, paramMean(4), inv_gamma_pdf, 0.15, 1;
ssigmazeta, paramMean(5), inv_gamma_pdf, 1, 1;
ssigmarhox, paramMean(6), inv_gamma_pdf, 1, 1;   
ssigmag, paramMean(7), inv_gamma_pdf, 0.5, 1;
ssigmaW, paramMean(8), inv_gamma_pdf, 0.5, 0.5;
ssigmaTFP, paramMean(9), inv_gamma_pdf, 0.5, 1;


ss2prime, paramMean(10), gamma_pdf, 4, 1;
ppsi, paramMean(11), beta_pdf, 0.5, 0.15;
xxiP, paramMean(12), beta_pdf, 0.5, 0.1;
iiotaP, paramMean(13), beta_pdf, 0.5, 0.15;
xxiW, paramMean(14), gamma_pdf, 1, 0.2;
iiotaW, paramMean(15), beta_pdf, 0.5, 0.15;

rrhoR, paramMean(16), beta_pdf, 0.6, 0.1;
pphiPi, paramMean(17), normal_pdf, 1.7, 0.3;
pphiX, paramMean(18), normal_pdf, 0, 0.3;
rrhoetaP, paramMean(19), beta_pdf, 0.6, 0.1;
rrhomuu, paramMean(20), beta_pdf, 0.4, 0.1;
rrhoupsilon, paramMean(21), beta_pdf, 0.6, 0.1;
rrhozeta, paramMean(22), beta_pdf, 0.6, 0.1;
rrhorhox, paramMean(23), beta_pdf, 0.6, 0.1;
rrhog, paramMean(24), beta_pdf, 0.6, 0.1;
rrhoTFP, paramMean(25), beta_pdf, 0.6, 0.1;



end;


// MCMC
estimation(datafile=estimationData,order=1,mode_compute=1, mh_nblocks=1, mh_replic=700000,
          moments_varendo, mh_drop=0.6,mh_jscale=0.4, plot_priors = 0) obsgX obsgC obsgI obsgG obsF obsS obsgWE obsPi obsFF;

stoch_simul(order=1, irf=20, nocorr, nograph ) obsgX obsgC obsgI obsgG obsF obsS obsgWE obsPi obsFF;
@#endif

@#if exer_option == 2
stoch_simul(order=1, irf = 0, noprint, nograph);

// Observables
varobs obsgC obsgI obsgG obsF obsS obsgWE obsPi obsFF;
calib_smoother(datafile=estimationData);

shock_decomposition(datafile=estimationData, parameter_set=calibration, nograph);
@#endif


