% Computes market-clearing beta and UI and associated distribution and 
% decision rules in steady state
%
% Daeha Cho, December, 2022

%----------------------------------------------------------------
% Compute approximation tools
%----------------------------------------------------------------
global bbetaLInit bInit

% Grids
computeGrids;

% Polynomials over grids (only if using polynomials to approximate conditional expectation)
if splineOpt == 0
	computePolynomials;
end

%----------------------------------------------------------------
% Compute initial guess of market-clearing capital stock using
% histogram approximation of distribution, from Young (2010)
%----------------------------------------------------------------

t0 = tic;
fprintf('Computing initial guess from histogram...\n')


rootInit0 = [bbetaLInit bInit]';
[rootInit,check] = broydn(@computeMCResidualHistogram,rootInit0,[1e-4,0,1]);
  if(check~=0)
            warning('compute_stst:broyerror','broydn not converged');
  end
[err] = computeMCResidualHistogram(rootInit);
bbetaL = rootInit(1);
bb = rootInit(2);

if strcmp(displayOpt,'iter-detailed') == 1
    fprintf('Done! Time to compute: %2.2f seconds \n\n',toc(t0))
    disp(['Errors: ' num2str(max(abs(err)))]);
end

%----------------------------------------------------------------
% Compute moments of histogram to use as initial guess for parametric distribution
%----------------------------------------------------------------

% Compute histogram
[~, mHistogram] = computeMCResidualHistogram(rootInit);

% Compute moments from histogram
mMomentsHistogram = zeros(nEpsilon,nMeasure);
aGridMoments = zeros(nEpsilon,nAssetsQuadrature,nMeasure);

%Terms used for computing PDF
for iEpsilon = 1 : nEpsilon
	
	% First moment (uncentered)
	mMomentsHistogram(iEpsilon,1) = sum(vAssetsGridFine' .* (mHistogram(iEpsilon,:) ./ ...
		sum(mHistogram(iEpsilon,:)))); %m^1
	aGridMoments(iEpsilon,:,1) = vAssetsGridQuadrature - mMomentsHistogram(iEpsilon,1); % (a-m1)
		
	% Higher order moments (centered)
	for iMoment = 2 : nMeasure
		mMomentsHistogram(iEpsilon,iMoment) = sum(((vAssetsGridFine' - mMomentsHistogram(iEpsilon,1)) .^ iMoment) .* ...
			(mHistogram(iEpsilon,:) ./ sum(mHistogram(iEpsilon,:)))); % m^i
		aGridMoments(iEpsilon,:,iMoment) = (vAssetsGridQuadrature' - mMomentsHistogram(iEpsilon,1)) .^ ...
			iMoment - mMomentsHistogram(iEpsilon,iMoment); % (a-m1)^i - m^i
	end	
	
end

% Mass at borrowing constraint
mHatHistogram = zeros(nEpsilon,1);
for iEpsilon = 1 : nEpsilon
    mHatHistogram(iEpsilon,1) = mHistogram(iEpsilon,1) / sum(mHistogram(iEpsilon,:));
end

%----------------------------------------------------------------
% Compute market-clearing beta and UI from parametric distribution
%----------------------------------------------------------------

t0 = tic; 
fprintf('Compute steady state from parametric family...\n')

% Solve for market clearing beta and UI
f = @(rootParameters) computeMCResidualPolynomials(rootParameters,mMomentsHistogram,aGridMoments,mHatHistogram);
options = optimoptions('fsolve','Display',displayOpt,'TolFun',1e-4);
if max(abs(f(rootInit))) > 1e-4
	[rootFinal,err2,exitflag] = fsolve(f,rootInit,options);
    bbetaL = rootFinal(1);
    bb = rootFinal(2);
end

% Return error if market clearing not solved
if exitflag < 1
    check = 1;
    return; 
end	

if strcmp(displayOpt,'iter-detailed') == 1
    fprintf('Done! Time to compute: %2.2f seconds \n\n',toc(t0))
    disp(['Errors: ' num2str(max(abs(err2)))]);
end

%----------------------------------------------------------------
% Compute other objects from steady state
%----------------------------------------------------------------

[err2,mCoefficients,mParameters,mMoments,mHat,mMomentsC] = ...
    computeMCResidualPolynomials(rootFinal,mMomentsHistogram,aGridMoments,mHatHistogram);

%----------------------------------------------------------------
% Compute other steady state values
%----------------------------------------------------------------
 cLSS = mMomentsC' * vEpsilonInvariant;
 cHSS = (cSS - (1-Hshare)*cLSS)/Hshare;
 ttauSS = (bb * wSS * uSS  + gSS + (RSS/(PiSS*muuSS)-1) * bgSS)/(wSS*nSS + bb*wSS*uSS);  
 aggImpatientWealth = vEpsilonInvariant' * mMoments(:,1);
 patientBond =  bgSS - (1-Hshare) * aggImpatientWealth; 
 aHSS = patientBond/Hshare; 

cL_TSS = ((1-ttauSS) * wSS * ( nSS +  bb * uSS) ) ;
cH_TSS = (cSS - (1-Hshare_T)*cL_TSS)/Hshare_T;

save steadyState_Decision_Distribution mCoefficients mParameters mMoments mHat mMomentsC...
    bbetaL bb cLSS cHSS ttauSS aHSS cL_TSS cH_TSS
