// Declare parameters and load in their values for HANKDynamics.mod

//----------------------------------------------------------------
// Preliminaries
//----------------------------------------------------------------

// Load in files containing parameters
economicParameters = load('economicParameters');
approximationParameters = load('approximationParameters');
steadyStateAggregates = load('steadyStateAggregates');
grids = load('grids');
polynomials = load('polynomials');

// Define economic parameters
parameters bbetaL ssigma aaBar bbetaH Hshare aalpha ddelta ss2prime rrhouk ppsi
	xxiP iiotaP ggamma rrhoxBar kkappa MMBar bb iiotaW xxiW FF rrhoR pphiPi pphiX
    rrhoetaP rrhomuu rrhoupsilon rrhoTFP rrhozeta rrhorhox rrhopiStar rrhog
    ssigmaetaP ssigmamuu ssigmaupsilon ssigmaTFP ssigmaR ssigmazeta ssigmarhox 
    ssigmapiStar ssigmag ssigmaW;


//Load in their values
@#define nEconomicParameters = 41
for iParam = 1 : @{nEconomicParameters}
	parameterName = deblank(M_.param_names(iParam,:));
	if isfield(economicParameters,parameterName)
		M_.params(iParam) = eval(['economicParameters.' parameterName]);
	end
end

// Define steady state aggregate variables
parameters RSS PiSS muuSS nSS fSS wSS mcSS rkSS kSS lambdaSS uSS uaSS thetaSS vSS rhoxSS
           etaPSS pStarSS h1PSS h2PSS DeltaPSS xSS bgSS ttauSS gSS gtildeSS dSS ySS invSS cSS cHSS cLSS;


//Load in their values
@#define nSteadyStateAggregates = 31
for iParam = 1 : @{nSteadyStateAggregates}
	parameterName = deblank(M_.param_names(@{nEconomicParameters}+iParam,:));
	if isfield(steadyStateAggregates,parameterName)
		M_.params(@{nEconomicParameters}+iParam) = eval(['steadyStateAggregates.' parameterName]);
	end
end

@#define nCounter = nEconomicParameters + nSteadyStateAggregates

// Define model option parameter
parameters modelOption_;

modelOption_ = modelOption; 

@#define nCounter = nCounter + 1

// Have to impose parameters used as counters directly
@#define nEpsilon = 2
@#define nAssets = 25
@#define nMeasure = 4
@#define nAssetsQuadrature = 20

@#define nState = nEpsilon * nAssets
@#define nStateQuadrature = nEpsilon * nAssetsQuadrature

// Epsilon transition matrix
@#for iEpsilon in 1 : nEpsilon
	@#for iEpsilonPrime in 1 : nEpsilon
		parameters epsilonTransitionSS_@{iEpsilon}_@{iEpsilonPrime};
	@#endfor
@#endfor

for iEpsilon = 1 : @{nEpsilon}
	for iEpsilonPrime = 1 : @{nEpsilon}
		M_.params(@{nCounter} + @{nEpsilon} * (iEpsilon - 1) + iEpsilonPrime) = ...
			steadyStateAggregates.mEpsilonTransitionSS(iEpsilon,iEpsilonPrime);
	end
end

@#define nCounter = nCounter + nEpsilon * nEpsilon

// Mass of invariant distrbution of idiosyncratic shocks
@#for iEpsilon in 1 : nEpsilon
	parameters epsilonMassSS_@{iEpsilon};
@#endfor

for iEpsilon = 1 : @{nEpsilon}
	M_.params(@{nCounter} + iEpsilon) = steadyStateAggregates.vEpsilonInvariant(iEpsilon);
end

@#define nCounter = nCounter + nEpsilon


// Define some of the approximation parameters 
parameters nEpsilon nAssets nState assetsMin assetsMax nAssetsFine nStateFine nAssetsQuadrature nStateQuadrature 
	nMeasure nMeasureCoefficients maxIterations tolerance dampening;

// Load in their values
@#define nApproximationParameters = 14
for iParam = 1 : @{nApproximationParameters}
	parameterName = deblank(M_.param_names(@{nCounter} + iParam,:));
	if isfield(approximationParameters,parameterName)
		M_.params(@{nCounter} + iParam) = eval(['approximationParameters.' parameterName]);
	end
end

@#define nCounter = nCounter + nApproximationParameters



//
//----------------------------------------------------------------
// Grids for approximating conditional expectation
//----------------------------------------------------------------

// Employment grids
@#for iEpsilon in 1 : nEpsilon
	parameters employmentGrid_@{iEpsilon};
@#endfor

for iEpsilon = 1 : @{nEpsilon}
	M_.params(@{nCounter} + iEpsilon) = grids.vEmploymentGrid(iEpsilon);
end

@#define nCounter = nCounter + nEpsilon

// Epsilon grids
@#for iEpsilon in 1 : nEpsilon
	parameters epsilonGrid_@{iEpsilon};
@#endfor

for iEpsilon = 1 : @{nEpsilon}
	M_.params(@{nCounter} + iEpsilon) = grids.vEpsilonGrid(iEpsilon);
end

@#define nCounter = nCounter + nEpsilon


//
// Assets
//

// Define the grids
@#for iAssets in 1 : nAssets
	parameters assetsGrid_@{iAssets};
@#endfor

// Assign values (must be in the same order that the parameters were declared)
for iAssets = 1 : @{nAssets}
	M_.params(@{nCounter} + iAssets) = grids.vAssetsGrid(iAssets);
end

@#define nCounter = nCounter + nAssets

//----------------------------------------------------------------
// Quadrature grid and weights
//----------------------------------------------------------------

// Define the parameters
@#for iAssets in 1 : nAssetsQuadrature
	parameters quadratureGrid_@{iAssets};
	parameters quadratureWeights_@{iAssets};
@#endfor

// Assign values
for iAssets = 1 : @{nAssetsQuadrature}
	M_.params(@{nCounter} + 2 * (iAssets - 1) + 1) = grids.vAssetsGridQuadrature(iAssets);
	M_.params(@{nCounter} + 2 * (iAssets - 1) + 2) = grids.vQuadratureWeights(iAssets);
end

@#define nCounter = nCounter + 2 * nAssetsQuadrature

//----------------------------------------------------------------
// Conditional expectation polynomials
//----------------------------------------------------------------

//
// Chebyshev polynomials
//

// Define the parameters
@#for iAssets in 1 : nAssets
	@#for iPower in 1 : nAssets
		parameters expectationPoly_@{iAssets}_@{iPower};
	@#endfor
@#endfor

// Assign values
for iAssets = 1 : @{nAssets}
	for iPower = 1 : @{nAssets}
		M_.params(@{nCounter} + @{nAssets} * (iAssets - 1) + iPower) = polynomials.vAssetsPoly(iAssets,iPower);
	end
end

@#define nCounter = nCounter + nAssets * nAssets

// 
// Squared terms in Chebyshev interpolation
//

// Define the parameters
@#for iAssets in 1 : nAssets
	parameters expectationPolySquared_@{iAssets};
@#endfor

// Assign the values
for iAssets = 1 : @{nAssets}
	M_.params(@{nCounter} + iAssets) = polynomials.vAssetsPolySquared(iAssets);	
end

@#define nCounter = nCounter + nAssets

//----------------------------------------------------------------
// Quadrature grid polynomials
//----------------------------------------------------------------

// Define the parameters
@#for iAssets in 1 : nAssetsQuadrature
	@#for iPower in 1 : nAssets
		parameters quadraturePoly_@{iAssets}_@{iPower};		
	@#endfor
@#endfor

// Assign values
for iAssets = 1 : @{nAssetsQuadrature}
	for iPower = 1 : @{nAssets}	
		M_.params(@{nCounter} + @{nAssets} * (iAssets - 1) + iPower) = polynomials.vAssetsPolyQuadrature(iAssets,iPower);		
	end	
end

@#define nCounter = nCounter + nAssetsQuadrature * nAssets

//----------------------------------------------------------------
// Borrowing constraint polynomials
//----------------------------------------------------------------

// Define the parameters
@#for iPower in 1 : nAssets
	parameters bcPoly_@{iPower};
@#endfor

// Assign values
for iPower = 1 : @{nAssets}
	M_.params(@{nCounter} + iPower) = polynomials.vAssetsPolyBC(iPower);
end