% Computes and analyzes steady state with no aggregate shocks
%
% Daeha Cho, December, 2022

clear all
close all
clc

setpath; 

oldFolder = cd('./Auxiliary Functions');

%----------------------------------------------------------------
% Set parameters
%----------------------------------------------------------------
load('HANKparam')
setParameters;

%----------------------------------------------------------------
% Compute Steady State
%----------------------------------------------------------------

% Solve for steady state distribution and decision rules
coreSteadyState;

% Compute decision rules along fine grid for analysis
rootParameters = [bbetaL bb]';
[~,mHistogram,mAssetsPrime,mConsumption] = computeMCResidualHistogram(rootParameters);

% Compute density along fine grid
mDistributionFine = zeros(nEpsilon,nAssetsFine);
for iEpsilon = 1 : nEpsilon

	% First moment (uncentered)
	mGridMoments = zeros(nAssetsFine,nMeasure);
	mGridMoments(:,1) = (vAssetsGridFine - mMoments(iEpsilon,1));
	
	% Higher order moments (centered)
	for iMoment = 2 : nMeasure
		mGridMoments(:,iMoment) = (vAssetsGridFine - mMoments(iEpsilon,1)) .^ iMoment - ...
			mMoments(iEpsilon,iMoment);
	end
	
	% Compute density away from borrowing constraint
	mDistributionFine(iEpsilon,:) = mParameters(iEpsilon,1) * exp(mGridMoments * ...
		mParameters(iEpsilon,2:nMeasure+1)');
				
end

%----------------------------------------------------------------
% Plot results (Figure 1)
%----------------------------------------------------------------


FigHandle = figure('Position', [150, 150, 1600, 600]);

set(0,'defaultlinelinewidth',2.5,'defaultLineMarkerSize',2);
subaxis(1,2,1,'spacing',0.1)
hold on
plot(vAssetsGridFine/ySS,mConsumption(1,:),'-','color',[8/255,62/255,118/255])
plot(vAssetsGridFine/ySS,mConsumption(2,:),'--','color',[178/255,34/255,34/255])
plot(vAssetsGridFine/ySS,vAssetsGridFine,'k--','linewidth',.5)
xlabel({'Liquid assets to output ratio'},'Interpreter','latex')
ylabel({'Consumption'},'Interpreter','latex')
xlim([aaBar 0.6])
title({'Decision rule'},'Interpreter','latex')
set(gca,'FontSize',15)
grid on
set(gcf,'color','w')
hold off

subaxis(1,2,2,'spacing',0.1);
hold on
plot(vAssetsGridFine/ySS,[mHat(1) mDistributionFine(1,2:end) ./ sum(mDistributionFine(1,2:end))]/sum([mHat(1) mDistributionFine(1,2:end) ./ sum(mDistributionFine(1,2:end))]),'-','color',[8/255,62/255,118/255])
plot(vAssetsGridFine/ySS,[mHat(2) mDistributionFine(2,2:end) ./ sum(mDistributionFine(1,2:end))]/sum([mHat(2) mDistributionFine(2,2:end) ./ sum(mDistributionFine(1,2:end))]),'--','color',[178/255,34/255,34/255])
hold off
xlabel({'Liquid assets to output ratio'},'Interpreter','latex')
ylabel({'Mass of households'},'Interpreter','latex')
grid on
xlim([0 0.2])
title({'Distribution'},'Interpreter','latex')
set(gca,'XTick',0:0.1:0.5);
set(gca,'FontSize',15)

lg = legend({'Unemployed','Employed'},'Interpreter','latex','location','best');
set(lg,'FontSize',15);
legend('boxoff');
set(gcf,'color','w')





%--------------------------------------------------------------------------
% Compute MPC, consumption/wealth distribution, wealth inequality (Table 2)
%--------------------------------------------------------------------------
vMPCL = zeros(nEpsilon,1);
for iEpsilon = 1 : nEpsilon
    
vMPCL(iEpsilon,1) =  ( sum((diff(mConsumption(iEpsilon,:))./((RSS/PiSS).*diff(vAssetsGridFine')))...
    .*mDistributionFine(iEpsilon,1:end-1)) +...
    (diff(mConsumption(iEpsilon,end-1:end))./((RSS/PiSS)*diff(vAssetsGridFine(end-1:end)')))...
    *mDistributionFine(iEpsilon,end) )/ sum(mDistributionFine(iEpsilon,:),2);
end

MPC = [vMPCL; MPCH];


consumptionShare = [((1-Hshare)*vEpsilonInvariant(1)*mMomentsC(1))/cSS;...
    ((1-Hshare)*vEpsilonInvariant(2)*mMomentsC(2))/cSS;...
    (Hshare*cHSS)/cSS]*100;

aggImpatientWealth = vEpsilonInvariant' * mMoments(:,1);
patientBond =  bgSS - (1-Hshare) * aggImpatientWealth; 
liquidWealthShare = [((1-Hshare)*vEpsilonInvariant(1)*mMoments(1,1))/bgSS;...
    ((1-Hshare)*vEpsilonInvariant(2)*mMoments(2,1))/bgSS;...
    patientBond/bgSS]*100;

giniIndexL = [gini([mDistributionFine(1,:)/sum(mDistributionFine(1,:),2)],[vAssetsGridFine], false);
    gini([mDistributionFine(2,:)/sum(mDistributionFine(2,:),2)],[vAssetsGridFine], false);
    gini(1,patientBond/Hshare, false)];


cd(oldFolder)