function [residual,mHistogramOptional,mAssetsPrimeOptional,mConsumptionOptional]...
    = computeMCResidualHistogram(rootInit)

%computes the difference between the model-implied MPC and the MPC target and 
%the difference between the model-implied EU consumption difference and the its target,
%using histogram to approximate the distribution;
%used to compute initial guess for parametric family
% 
% Daeha Cho, December, 2022

% Declare global variables
global bbetaL MPCH ssigma Hshare aaBar bb ttauSS uSS nSS gSS bgSS muuSS...
    mEpsilonTransitionSS nEpsilon nAssets vAssetsGrid vEpsilonGrid mEmploymentGrid mAssetsGrid ...
	vAssetsPoly vAssetsPolySquared wSS RSS PiSS kSS targetMPC cUOvercEMean mMomentsHistogramC ...
    maxIterations tolerance dampening vAssetsPolyFine vAssetsGridFine ...
	mEmploymentGridFine mAssetsGridFine nAssetsFine nStateFine splineOpt

% Initialize
residual = zeros(length(rootInit),1);

% Label
bbetaL = rootInit(1);
bb = rootInit(2);


%----------------------------------------------------------------
% Compute individual decisions
%----------------------------------------------------------------

ttauSS = (bb * wSS * uSS  + gSS + (RSS/(PiSS*muuSS)-1) * bgSS)/(wSS*nSS + bb*wSS*uSS);  

if splineOpt == 0	% approximate conditional expectation function using polynomials

	% Initialize coefficients using rule of thumb savings rule (a=a')
	mGridInit = log(bbetaL * (RSS/(PiSS*muuSS^ssigma)) * (((1 - ttauSS) * wSS * (bb * (1 - mEmploymentGrid) + mEmploymentGrid.* vEpsilonGrid) + ...
		(RSS/(PiSS*muuSS)-1) * mAssetsGrid) .^ (-ssigma))); 
	mCoefficients = zeros(nEpsilon,nAssets);
	for iEpsilon = 1:nEpsilon	
		vCoefficients = sum(vAssetsPoly' .* (ones(nAssets,1) * mGridInit(iEpsilon,:)),2); 
		mCoefficients(iEpsilon,:) = (vCoefficients ./ vAssetsPolySquared)';
	end

	% Iterate
	err = 100; iteration = 1;
	while err > tolerance && iteration <= maxIterations

		mCoefficientsNew = updateCoefficients_polynomials(mCoefficients);
		err = max(abs(mCoefficientsNew(:) - mCoefficients(:)));
		iteration = iteration + 1;
		mCoefficients = dampening * mCoefficients + (1 - dampening) * mCoefficientsNew;

	end
	
else	% approximate savings decision using linear splines

	mAssetsPrime = mAssetsGrid; 
	
	% Iterate
	err = 100; iteration = 1;
	while err > tolerance && iteration <= maxIterations
		mAssetsPrimeNew = updateCoefficients_splines(mAssetsPrime);
		err = max(abs(mAssetsPrimeNew(:) - mAssetsPrime(:)));
		iteration = iteration + 1;
		mAssetsPrime = dampening * mAssetsPrime + (1 - dampening) * mAssetsPrimeNew;
	end

end

%----------------------------------------------------------------
% Compute histogram approximation of stationary distribution
%----------------------------------------------------------------

%%%
% Compute policies over histogram grid	
%%%

if splineOpt == 0

	% Compute decision rules along fine grid
	mConditionalExpectation = exp(mCoefficients * vAssetsPolyFine');

	% Compute savings policy in period t: mAssetsPrimeFine = a'(mAssetsGridFine)
	mAssetsPrimeStar = (1 - ttauSS) * wSS * (bb * (1 - mEmploymentGridFine) + mEmploymentGridFine.* vEpsilonGrid) + ...
		(RSS/(PiSS*muuSS)) * mAssetsGridFine - (mConditionalExpectation .^ (-1 / ssigma));
	mAssetsPrimeFine = max(mAssetsPrimeStar,aaBar * ones(nEpsilon,nAssetsFine));
		
else	% linearly interpolate savings rule over fine grid

	% Compute weights
	[vIndicesBelow,vIndicesAbove,vWeightBelow,vWeightAbove] = computeLinearWeights(vAssetsGrid,vAssetsGridFine);
		
	% Compute savings policy in period t using linear interpolation: mAssetsPrimeFine(1,1) = a'(vAssetsGridFine(1))
	mAssetsPrimeFine = mAssetsPrime(:,vIndicesBelow) .* repmat(vWeightBelow',nEpsilon,1) + ...
		mAssetsPrime(:,vIndicesAbove) .* repmat(vWeightAbove',nEpsilon,1);
	
end

% Compute consumption in period t: mConsumptionFine = c(mAssetsGridFine)
mConsumptionFine = (1 - ttauSS) * wSS * (bb * (1 - mEmploymentGridFine) + mEmploymentGridFine.* vEpsilonGrid) + ...
	(RSS/(PiSS*muuSS)) * mAssetsGridFine - mAssetsPrimeFine;
	
%%%
% Compute transition matrix associated with policy rules
%%%

% Compute weighting matrices
[vIndicesBelow,vIndicesAbove,vWeightBelow,vWeightAbove] = computeLinearWeights(vAssetsGridFine,mAssetsPrimeFine(:));
	
% Compute transition matrix for assets over full grid
mTransitionAbove = zeros(nStateFine,nAssetsFine);
mTransitionBelow = zeros(nStateFine,nAssetsFine);
for a = 1:nAssetsFine
	mTransitionBelow(vIndicesBelow == a,a) = vWeightBelow(vIndicesBelow == a);
	mTransitionAbove(vIndicesAbove == a,a) = vWeightAbove(vIndicesAbove == a);
end
mAssetsTransition = kron(mTransitionBelow + mTransitionAbove,ones(1,nEpsilon));

% Compute transition matrix for idiosyncratic shocks over full grid
mEpsilonTransitionHistogram = repmat(mEpsilonTransitionSS,nAssetsFine);

% Compute full transition matrix
mTransition = sparse(mAssetsTransition .* mEpsilonTransitionHistogram);

% Compute invariant histogram by iteration
errHistogram = 100;	iterationHistogram = 0;
vHistogram = ones(nStateFine,1) ./ nStateFine; %initial guess for histogram
while errHistogram > 1e-12 && iterationHistogram < 1e4
	
	vHistogramNew = mTransition' * vHistogram;
	errHistogram = max(abs(vHistogramNew - vHistogram)); iterationHistogram = iterationHistogram + 1;
	vHistogram = vHistogramNew;
	
end

% Expand histogram matrix 
mHistogram = reshape(full(vHistogramNew),nEpsilon,nAssetsFine);


MPCL = 0;
for iEpsilon = 1 : nEpsilon
MPCL = MPCL + sum((diff(mConsumptionFine(iEpsilon,:))./((RSS/(PiSS*muuSS)).*diff(vAssetsGridFine')))...
    .*mHistogram(iEpsilon,1:end-1)) +...
    (diff(mConsumptionFine(iEpsilon,end-1:end))./((RSS/(PiSS*muuSS))*diff(vAssetsGridFine(end-1:end)')))...
    *mHistogram(iEpsilon,end);
end   
residual(1) = (1-Hshare)*MPCL + Hshare * MPCH - targetMPC;

%Compute consumption differential between employed and unemployed 
mMomentsHistogramC = sum(mConsumptionFine.*(mHistogram./sum(mHistogram,2)),2);
consumptionUnemployedMean = mMomentsHistogramC(1);
consumptionEmployedMean = mMomentsHistogramC(2);

residual(2) = (consumptionUnemployedMean/consumptionEmployedMean) - cUOvercEMean;

if nargout > 1 

    mHistogramOptional = mHistogram;
	
    if nargout > 2
        mAssetsPrimeOptional = mAssetsPrimeFine;
        mConsumptionOptional = mConsumptionFine;
    end
	
end