function [residual,mCoefficientsOptional,mParametersOptional,mMomentsOptional,mHatOptional, mMomentsCOptional] = ...
	computeMCResidualPolynomials(rootInit,mMoments,aGridMoments,mHat)

%computes the difference between the model-implied MPC and the MPC target and 
%the difference between the model-implied EU consumption difference and the its target,
%using parametric family to approximate the distribution
%
% Daeha Cho, December, 2022

% Declare global variables
global bbetaL MPCH ssigma Hshare aaBar bb ttauSS uSS nSS gSS bgSS muuSS...
    mEpsilonTransitionSS vEpsilonGrid vEmploymentGrid...
	nEpsilon nAssets vAssetsGrid mEmploymentGrid mAssetsGrid assetsMin assetsMax ...
	vAssetsPoly vAssetsPolySquared  wSS RSS PiSS kSS targetMPC cUOvercEMean mMomentsHistogramC...
    maxIterations tolerance dampening  ...
	vAssetsPolyQuadrature vAssetsGridQuadrature mEmploymentGridQuadrature mAssetsGridQuadrature nAssetsQuadrature ...
	nStateQuadrature vQuadratureWeights vEpsilonInvariant nMeasure splineOpt vAssetsPolyBC...
    vAssetsGridFine nAssetsFine


% Label
bbetaL = rootInit(1);
bb = rootInit(2);
mMomentsHistogram = mMoments;
mHatHistogram = mHat;

%----------------------------------------------------------------
% Set error tolerance & max iteration depending on use
%----------------------------------------------------------------

if nargout == 1 
    err1 = tolerance;
    err2 = 1e-4;
    tol2 = 200;
elseif nargout > 1 
    err1 = 1e-8;
    err2 = 1e-6;
    tol2 = 500;
else
    error('Check # of inputs & outputs for computeMCResidualPolynomials');
end

%----------------------------------------------------------------
% Compute individual decisions for given beta and UI
%----------------------------------------------------------------
ttauSS = (bb * wSS * uSS  + gSS + (RSS/(PiSS*muuSS)-1) * bgSS)/(wSS*nSS + bb*wSS*uSS);  

if splineOpt == 0	% approximate conditional expectation function using polynomials

	% Initialize coefficients using rule of thumb savings rule (a=a')
	mGridInit = log(bbetaL * (RSS/(PiSS*muuSS^ssigma)) * (((1 - ttauSS) * wSS * (bb * (1 - mEmploymentGrid) + mEmploymentGrid.* vEpsilonGrid) + ...
		(RSS/(PiSS*muuSS)-1) * mAssetsGrid) .^ (-ssigma))); 
	mCoefficients = zeros(nEpsilon,nAssets);    
	for iEpsilon = 1:nEpsilon	
		vCoefficients = sum(vAssetsPoly' .* (ones(nAssets,1) * mGridInit(iEpsilon,:)),2); 
		mCoefficients(iEpsilon,:) = (vCoefficients ./ vAssetsPolySquared)';
	end

	% Iterate
	err = 100; iteration = 1;
	while err > tolerance && iteration <= maxIterations

		mCoefficientsNew = updateCoefficients_polynomials(mCoefficients);
		err = max(abs(mCoefficientsNew(:) - mCoefficients(:)));
		iteration = iteration + 1;
		mCoefficients = dampening * mCoefficients + (1 - dampening) * mCoefficientsNew;

	end
	
	mCoefficientsOptional = mCoefficients;
	
else	% approximate savings decision using linear splines
	% Initialize coefficients
	mAssetsPrime = mAssetsGrid;
	
	% Iterate
	err = 100; iteration = 1;
	while err > tolerance && iteration <= maxIterations
		mAssetsPrimeNew = updateCoefficients_splines(mAssetsPrime);
		err = max(abs(mAssetsPrimeNew(:) - mAssetsPrime(:)));
		iteration = iteration + 1;
		mAssetsPrime = dampening * mAssetsPrime + (1 - dampening) * mAssetsPrimeNew;
	end
	
	mCoefficientsOptional = mAssetsPrime;

end

%----------------------------------------------------------------
% Compute policies over quadrature grid for integration
%----------------------------------------------------------------

if splineOpt == 0

	% Compute conditional expectation
	mConditionalExpectation = exp(mCoefficients * vAssetsPolyQuadrature');

	% Compute savings policy
	mAssetsPrimeStar = (1 - ttauSS) * wSS * (bb * (1 - mEmploymentGridQuadrature) + mEmploymentGridQuadrature.* vEpsilonGrid) + ...
		(RSS/(PiSS*muuSS)) * mAssetsGridQuadrature - (mConditionalExpectation .^ (-1 / ssigma));
	mAssetsPrimeQuadrature = max(mAssetsPrimeStar,aaBar * ones(nEpsilon,nAssetsQuadrature));

else

	% Compute weights
	[vIndicesBelow,vIndicesAbove,vWeightBelow,vWeightAbove] = computeLinearWeights(vAssetsGrid,vAssetsGridQuadrature);
		
	% Linear interpolation
	mAssetsPrimeQuadrature = mAssetsPrime(:,vIndicesBelow) .* repmat(vWeightBelow',nEpsilon,1) + ...
		mAssetsPrime(:,vIndicesAbove) .* repmat(vWeightAbove',nEpsilon,1);
	
end

% Consumption in period t: mConsumptionQuadrature = c(mAssetsGridQuadrature)
mConsumptionQuadrature = (1 - ttauSS) * wSS * (bb * (1 - mEmploymentGridQuadrature) + mEmploymentGridQuadrature.* vEpsilonGrid) + ...
	(RSS/(PiSS*muuSS)) * mAssetsGridQuadrature - mAssetsPrimeQuadrature;


%----------------------------------------------------------------
% Compute policies at borrowing constraint for integration
%----------------------------------------------------------------

if splineOpt == 0

	% Compute conditional expectation
	mConditionalExpectation = exp(mCoefficients * vAssetsPolyBC');

	% Compute savings policy
	mAssetsPrimeStar = (1 - ttauSS) * wSS * (bb * (1 - vEmploymentGrid) + vEmploymentGrid.* vEpsilonGrid) + ...
		(RSS/(PiSS*muuSS)) * assetsMin - (mConditionalExpectation .^ (-1 / ssigma));
	mAssetsPrimeBC = max(mAssetsPrimeStar,aaBar * ones(nEpsilon,1));
		
else

	% Compute weights
	[vIndicesBelow,vIndicesAbove,vWeightBelow,vWeightAbove] = computeLinearWeights(vAssetsGrid,assetsMin);
		
	% Linear interpolation
	mAssetsPrimeBC = mAssetsPrime(:,vIndicesBelow) .* repmat(vWeightBelow',nEpsilon,1) + ...
		mAssetsPrime(:,vIndicesAbove) .* repmat(vWeightAbove',nEpsilon,1);
	
end

% Consumption in period t: mConsumptionBC = c(assetsMin)
mConsumptionBC = (1 - ttauSS) * wSS * (bb * (1 - vEmploymentGrid) + vEmploymentGrid.* vEpsilonGrid) + ...
	(RSS/(PiSS*muuSS)) * assetsMin - mAssetsPrimeBC;

%----------------------------------------------------------------
% Compute stationary distribution from these decision rules
%----------------------------------------------------------------

% Initialize iteration
err = 100; iteration = 1; 
options = optimoptions(@fminunc,'Algorithm','quasi-newton','Display','notify-detailed',...
	'MaxFunEvals',50000,'TolFun',1e-12,'GradObj','on','MaxIter',1000);
%{ For older versions of MATLAB:
options = optimset('LargeScale','off','Display','notify-detailed',...
	'MaxFunEvals',50000,'TolFun',1e-12,'GradObj','on','MaxIter',1000);
%}

% Iteration
while err > err2 && iteration <= tol2
	
	%%%
	% Update density away from borrowing constraint
	%%%

	% Compute parameters of the distribution by minimization
	mParameters = zeros(nEpsilon,nMeasure+1);
	for iEpsilon = 1 : nEpsilon
		objectiveFunction = @(vParametersTilde) parametersResidual(vParametersTilde,squeeze(aGridMoments(iEpsilon,:,:)));
		[vParameters,normalization] = fminunc(objectiveFunction,zeros(nMeasure,1),options);
		mParameters(iEpsilon,:) = [1 / normalization; vParameters]; 
	end
	
	% Compute moments and centered moments in period t+1 using those in period t 
	mMomentsNew = zeros(nEpsilon,nMeasure);
	aGridMomentsNew = zeros(nEpsilon,nAssetsQuadrature,nMeasure);
	
	for iEpsilon = 1 : nEpsilon
		
		% Compute first moment (uncentered)
		mMomentsNew(iEpsilon,1) = 0;
		for iEpsilonTilde = 1 : nEpsilon
			mMomentsNew(iEpsilon,1) = mMomentsNew(iEpsilon,1) + (1 - mHat(iEpsilonTilde,1)) * vEpsilonInvariant(iEpsilonTilde) * mEpsilonTransitionSS(...
				iEpsilonTilde,iEpsilon) * mParameters(iEpsilonTilde,1) * vQuadratureWeights' * (mAssetsPrimeQuadrature(iEpsilonTilde,:)' .* ...
				exp(squeeze(aGridMoments(iEpsilonTilde,:,:)) * mParameters(iEpsilonTilde,2:nMeasure+1)')) + mHat(iEpsilonTilde,1) * ...
				vEpsilonInvariant(iEpsilonTilde) * mEpsilonTransitionSS(iEpsilonTilde,iEpsilon) * mAssetsPrimeBC(iEpsilonTilde,1);	
		end
		
		mMomentsNew(iEpsilon,1) = mMomentsNew(iEpsilon,1) / vEpsilonInvariant(iEpsilon); %m^1 in t+1
		aGridMomentsNew(iEpsilon,:,1) = vAssetsGridQuadrature - mMomentsNew(iEpsilon,1); %(a-m^1) in t+1
		
		% Compute higher order moments (centered)
		for iMoment = 2 : nMeasure
		
			mMomentsNew(iEpsilon,iMoment) = 0;
			
			for iEpsilonTilde = 1 : nEpsilon
				mMomentsNew(iEpsilon,iMoment) = mMomentsNew(iEpsilon,iMoment) + (1 - mHat(iEpsilonTilde,1)) * vEpsilonInvariant(iEpsilonTilde) * mEpsilonTransitionSS(...
					iEpsilonTilde,iEpsilon) * mParameters(iEpsilonTilde,1) * vQuadratureWeights' * (((mAssetsPrimeQuadrature(iEpsilonTilde,:)' - ...
					mMomentsNew(iEpsilon,1)) .^ iMoment) .* exp(squeeze(aGridMoments(iEpsilonTilde,:,:)) * ...
					mParameters(iEpsilonTilde,2:nMeasure+1)')) + mHat(iEpsilonTilde,1) * vEpsilonInvariant(iEpsilonTilde) * ...
					mEpsilonTransitionSS(iEpsilonTilde,iEpsilon) * ((mAssetsPrimeBC(iEpsilonTilde,1) - mMomentsNew(iEpsilon,1)) .^ iMoment);
					
			end
			
			mMomentsNew(iEpsilon,iMoment) = mMomentsNew(iEpsilon,iMoment) / vEpsilonInvariant(iEpsilon);%m^i in t+1
			aGridMomentsNew(iEpsilon,:,iMoment) = (vAssetsGridQuadrature' - mMomentsNew(iEpsilon,1)) .^ iMoment - ...
				mMomentsNew(iEpsilon,iMoment); % (a-m^1)^i - m^i
				
		end
		
	end

	%%%
	% Update mass at borrowing constraint
    % i.e, law of motion for mass at borrowing constraint
	%%%
	
	mHatNew = zeros(nEpsilon,1);
	
	for iEpsilon = 1 : nEpsilon
	
		for iEpsilonTilde = 1 : nEpsilon
			mHatNew(iEpsilon,1) = mHatNew(iEpsilon,1) + (1 - mHat(iEpsilonTilde,1)) * vEpsilonInvariant(iEpsilonTilde) * ...
				mEpsilonTransitionSS(iEpsilonTilde,iEpsilon) * mParameters(iEpsilonTilde,1) * vQuadratureWeights' * ...
				((mAssetsPrimeQuadrature(iEpsilonTilde,:)' <= aaBar + 1e-8) .* exp(squeeze(aGridMoments(iEpsilonTilde,:,:)) * mParameters(iEpsilonTilde,2:nMeasure+1)')) + ...
				mHat(iEpsilonTilde,1) * vEpsilonInvariant(iEpsilonTilde) * mEpsilonTransitionSS(iEpsilonTilde,iEpsilon) * ...
				(mAssetsPrimeBC(iEpsilonTilde,1) <= aaBar + 1e-8);
				
		end
			
		mHatNew(iEpsilon,1) = mHatNew(iEpsilon,1) / vEpsilonInvariant(iEpsilon); %mhat in t+1
	
   end
	
	%%%
	% Update iteration
	%%%
	
	err = max([max(abs(mMomentsNew(:) - mMoments(:))),max(abs(mHatNew(:) - mHat(:)))]);
	iteration = iteration + 1;
	mMoments = mMomentsNew;
	aGridMoments = aGridMomentsNew;
	mHat = mHatNew;
	
end

%----------------------------------------------------------------
% Compute residuals
%----------------------------------------------------------------

%%%%% Compute MPCs %%%%%
[~,~,~,mConsumption]  = computeMCResidualHistogram(rootInit);

% Compute density along fine grid
mDistributionFine = zeros(nEpsilon,nAssetsFine);
for iEpsilon = 1 : nEpsilon

	% First moment (uncentered)
	mGridMoments = zeros(nAssetsFine,nMeasure);
	mGridMoments(:,1) = (vAssetsGridFine - mMoments(iEpsilon,1));
	
	% Higher order moments (centered)
	for iMoment = 2 : nMeasure
		mGridMoments(:,iMoment) = (vAssetsGridFine - mMoments(iEpsilon,1)) .^ iMoment - ...
			mMoments(iEpsilon,iMoment);
	end
	
	% Compute density away from borrowing constraint
	mDistributionFine(iEpsilon,:) = mParameters(iEpsilon,1) * exp(mGridMoments * ...
		mParameters(iEpsilon,2:nMeasure+1)');
				
end

vMPCL = zeros(nEpsilon,1);
for iEpsilon = 1 : nEpsilon
    
vMPCL(iEpsilon,1) =  ( sum((diff(mConsumption(iEpsilon,:))./((RSS/PiSS).*diff(vAssetsGridFine')))...
    .*mDistributionFine(iEpsilon,1:end-1)) +...
    (diff(mConsumption(iEpsilon,end-1:end))./((RSS/PiSS)*diff(vAssetsGridFine(end-1:end)')))...
    *mDistributionFine(iEpsilon,end) )/ sum(mDistributionFine(iEpsilon,:),2);
end
MPC = Hshare * MPCH + (1-Hshare) * (vEpsilonInvariant' * vMPCL); 

residual(1) = MPC - targetMPC;

%%%%% Compute the mean consumption ratio %%%%%
mMomentsC = zeros(nEpsilon,1);
for iEpsilon = 1 : nEpsilon 
    mMomentsC(iEpsilon,1) =  mParameters(iEpsilon,1) * vQuadratureWeights' * (mConsumptionQuadrature(iEpsilon,:)' .* ...
				exp(squeeze(aGridMoments(iEpsilon,:,:)) * mParameters(iEpsilon,2:nMeasure+1)'));	
end

consumptionUnemployedMean = mMomentsC(1);
consumptionEmployedMean = mMomentsC(2:end);

residual(2) = (consumptionUnemployedMean/consumptionEmployedMean) - cUOvercEMean;


% Also return optional outputs if requested
if nargout > 2

    mParametersOptional = mParameters;
    mMomentsOptional = mMoments;
	mHatOptional = mHat;
    mMomentsCOptional = mMomentsC;
	
end
