function mCoefficientsNew = updateCoefficients_polynomials(mCoefficients)

% Updates polynomial coefficients approximating the conditional expectation function in steady state
% 
% Inputs
%   (1) mCoefficients: nEpsilon x nAssets matrix, storing previous iteration's coefficients
%
% Outputs
%   (1) mCoefficientsNew: nEpsilon x nAssets matrix, storing updated coefficients
% 
% Daeha Cho, December, 2022

% Declare global variables


global bbetaL ssigma aaBar bb ttauSS uSS nSS gSS bgSS muuSS mEpsilonTransitionSS ...
	nEpsilon nAssets nState assetsMin assetsMax ...
	vEpsilonGrid mEmploymentGrid mAssetsGrid ...
	vAssetsPoly vAssetsPolySquared wSS RSS PiSS mEmploymentPrimeGrid

%---------------------------------------------------------------
% Compute current period's savings policy function
%---------------------------------------------------------------

ttauSS = (bb * wSS * uSS  + gSS + (RSS/(PiSS*muuSS)-1) * bgSS)/(wSS*nSS + bb*wSS*uSS);  

% Compute conditional expectation 
mConditionalExpectation = exp(mCoefficients * vAssetsPoly'); 
% Compute target saving
mAssetsPrimeStar = (1 - ttauSS) * wSS * (bb * (1 - mEmploymentGrid) + mEmploymentGrid.* vEpsilonGrid) + (RSS/(PiSS*muuSS)) * mAssetsGrid - ...
	(mConditionalExpectation .^ (-1 / ssigma));

% Compute actual saving
mAssetsPrime = max(mAssetsPrimeStar,aaBar * ones(nEpsilon,nAssets));

mAssetsPrimeGrid = repmat(reshape(mAssetsPrime,1,nState),[nEpsilon 1]);  % nEpsilon by nState 

% Compute next period's polynomials
mAssetsPrimeZeros = scaleDown(mAssetsPrime,assetsMin,assetsMax);
% computing polynomials for all asset grids in time t+1
mPolyAssetsPrime = computeChebyshev(nAssets,reshape(mAssetsPrimeZeros,nState,1)); % nState by nAssets

%---------------------------------------------------------------
% Compute next period's savings policy function
%---------------------------------------------------------------

% Compute conditional expectation
mConditionalExpectationPrime = exp(mCoefficients * mPolyAssetsPrime'); %psi_{t+1}(epsilon,a'): nEpsilon by nState

% Compute target saving
mAssetsPrimePrimeStar = (1 - ttauSS) * wSS * (bb * (1 - mEmploymentPrimeGrid) + mEmploymentPrimeGrid.* vEpsilonGrid) + (RSS/(PiSS*muuSS)) * mAssetsPrimeGrid - ...
	(mConditionalExpectationPrime .^ (-1 / ssigma));

% Compute actual savings
mAssetsPrimePrimeGrid = max(mAssetsPrimePrimeStar,aaBar*ones(nEpsilon,nEpsilon*nAssets));

%---------------------------------------------------------------
% Update conditional expectation function
%---------------------------------------------------------------

% Compute new conditional expectation function
mConsumptionPrime = (1 - ttauSS) * wSS * (bb * (1 - mEmploymentPrimeGrid) + mEmploymentPrimeGrid.* vEpsilonGrid) + (RSS/(PiSS*muuSS)) * ...
	mAssetsPrimeGrid - mAssetsPrimePrimeGrid; %nEpsilon by nState:c_{t+1}
aConditionalExpectationTilde = reshape(bbetaL * mEpsilonTransitionSS * ((RSS/(PiSS*muuSS^ssigma)) * (mConsumptionPrime .^ (-ssigma))),...
	nEpsilon,nEpsilon,nAssets);

% Extract the relevant entries: 
mConditionalExpectation = zeros(nEpsilon,nAssets);
for iEpsilon = 1:nEpsilon
	mConditionalExpectation(iEpsilon,:) = aConditionalExpectationTilde(iEpsilon,iEpsilon,:);
end

% Update the coefficients
mCoefficientsNew = zeros(nEpsilon,nAssets);
for iEpsilon = 1:nEpsilon
	vCoefficients = sum(vAssetsPoly' .* (ones(nAssets,1) * log(mConditionalExpectation(iEpsilon,:))),2);
	mCoefficientsNew(iEpsilon,:) = (vCoefficients ./ vAssetsPolySquared)';
end

