% This script generates the business cycle dynamics and impulse responses of the estimated models 
% and performs the robustness checks. 
%
% Daeha Cho, December, 2022

clear all
close all
clc

setpath;

cd('./Auxiliary Functions');

%----------------------------------------------------------------
% Set parameters
%----------------------------------------------------------------
load('HANKparam')
setParameters;

%----------------------------------------------------------------
% Compute Steady State
%----------------------------------------------------------------
coreSteadyState;


%----------------------------------------------------------------
% Compute approximation tools
%----------------------------------------------------------------

% Grids
computeGrids;

% Polynomials over grids
computePolynomials;

%----------------------------------------------------------------
% Save parameters in .mat files to import into Dynare 
%----------------------------------------------------------------

load('steadyState_Decision_Distribution');

    
% Economic parameters
save economicParameters.mat bbetaL ssigma aaBar bbetaH Hshare Hshare_T aalpha ddelta ss2prime rrhouk ppsi ...
     xxiP iiotaP ggamma rrhoxBar kkappa MMBar bb iiotaW xxiW FF rrhoR pphiPi pphiX vEpsilonGrid ...
     rrhoetaP rrhomuu rrhoupsilon rrhoTFP rrhozeta rrhorhox rrhopiStar rrhog... 
     ssigmaetaP ssigmamuu ssigmaupsilon ssigmaTFP ssigmaR ssigmazeta ssigmarhox...
     ssigmapiStar ssigmag ssigmaW;
    
% Steady state aggregate variables
save steadyStateAggregates.mat RSS PiSS muuSS nSS fSS rhoxSS wSS etaPSS DeltaPSS pStarSS h1PSS h2PSS...
     mcSS rkSS kSS lambdaSS uSS uaSS thetaSS vSS xSS bgSS ttauSS dSS ySS invSS gSS gtildeSS cSS cHSS cLSS cH_TSS cL_TSS aHSS...
     mEpsilonTransitionSS vEpsilonInvariant

% Approximation parameters
save approximationParameters.mat nEpsilon nAssets nState assetsMin assetsMax nAssetsFine nStateFine nAssetsQuadrature nStateQuadrature ...
	 nMeasure nMeasureCoefficients maxIterations tolerance dampening

% Grids
save grids.mat vAssetsGridZeros vAssetsGrid vEmploymentGrid vEpsilonGrid mEmploymentGrid mAssetsGrid mEmploymentPrimeGrid vAssetsGridFine ...
	 vAssetsGridFineZeros mEmploymentGridFine mAssetsGridFine mEmploymentPrimeGridFine vQuadratureWeights ...
	 vAssetsGridQuadratureZeros vAssetsGridQuadrature mEmploymentGridQuadrature mAssetsGridQuadrature
		
% Polynomials
save polynomials.mat vAssetsPoly vAssetsPolySquared vAssetsPolyFine vAssetsPolyQuadrature vAssetsPolyBC


%----------------------------------------------------------------
% Run Dynare to extract historical shocks from the estimated HANK  
%----------------------------------------------------------------

dynare HANKDynamicsShocks.mod -Dmodel_option=1 -DnMeasure_option=4

x_pos = strmatch('obsgX', M_.endo_names, 'exact');
c_pos = strmatch('obsgC', M_.endo_names, 'exact');
i_pos = strmatch('obsgI', M_.endo_names, 'exact');
pi_pos = strmatch('obsPi', M_.endo_names, 'exact');
f_pos = strmatch('obsF', M_.endo_names, 'exact');
s_pos = strmatch('obsS', M_.endo_names, 'exact');
u_pos = strmatch('obsU', M_.endo_names, 'exact');
v_pos = strmatch('vHat', M_.endo_names, 'exact');


% Volatility labor market variables: HANK vs. data (Table A.3)   
load('estimationData')
stdTable_HANKmodel_Labor = [sqrt(oo_.var(f_pos,f_pos)) sqrt(oo_.var(s_pos,s_pos)) sqrt(oo_.var(u_pos,u_pos)) sqrt(oo_.var(v_pos,v_pos));...
                      std(obsF) std(obsS) std(obsU) std(obsV)];   

%Information used to produce Table 7
stdTable_HANKmodel_T7 = [sqrt(oo_.var(c_pos,c_pos)) sqrt(oo_.var(i_pos,i_pos)) sqrt(oo_.var(pi_pos,pi_pos))...
                        sqrt(oo_.var(f_pos,f_pos)) sqrt(oo_.var(s_pos,s_pos));...
                      std(obsgC) std(obsgI) std(obsPi) std(obsF) std(obsS)];  
%save stdTable_HANKmodel_T7 stdTable_HANKmodel_T7                  
                     

% Extracting smoothed shocks and smoothed variables
smoothedVariables = oo_.SmoothedVariables;
save smoothedVariables smoothedVariables

smoothedShocks = [oo_.SmoothedShocks.epsetaP oo_.SmoothedShocks.epsmuu...
oo_.SmoothedShocks.epsupsilon oo_.SmoothedShocks.epsTFP oo_.SmoothedShocks.epsR...
oo_.SmoothedShocks.epszeta oo_.SmoothedShocks.epsrhox oo_.SmoothedShocks.epsg...
oo_.SmoothedShocks.epsW];

save smoothedShocks smoothedShocks


% Extracting the 1st element of smoothed variables 
firstSmoothedValues = zeros(length(M_.endo_names),1);
for ivar = 1:length(M_.endo_names)
        value = getfield(oo_.SmoothedVariables,M_.endo_names(ivar,:));   
        firstSmoothedValues(ivar) = value(1);
end

save firstSmoothedValues firstSmoothedValues

clear smoothedShocks firstSmoothedValues

%----------------------------------------------------------------
% Prepare counterfactuals
%----------------------------------------------------------------
load('smoothedShocks')
load('firstSmoothedValues')

meanCGrowth = 0.53;
meanIGrowth = 0.45;
meanGDPGrowth = 0.47;


shockNames = {'epsetaP', 'epsmuu', 'epsupsilon', 'epsTFP', 'epsR', 'epszeta', 'epsrhox', 'epsg', 'epsW'};

                
                      
%------------------------------------------------------------------------  
% Compute moments of aggregate variables subject to non-preference shocks
% Produces Figure 2, 6 and Table 4
%------------------------------------------------------------------------

load('smoothedShocks')
smoothedShocks(:,find(strcmp(shockNames, 'epszeta'))) = zeros(size(smoothedShocks,1),1);   


%Feed the smoothed shocks into HANK, RANK, and TANK and generate the model-implied time-series  
dynare counterFactuals.mod -Dmodel_option=1 -DnMeasure_option=4

gdpGrowth_HANK = meanGDPGrowth + outcome(:,strmatch('obsgX', M_.endo_names, 'exact'));
cGrowth_HANK = meanCGrowth + outcome(:,strmatch('obsgC', M_.endo_names, 'exact'));
iGrowth_HANK = meanIGrowth + outcome(:,strmatch('obsgI', M_.endo_names, 'exact'));
gdpCycle_HANK = regcyc(cumsum(gdpGrowth_HANK)); 
gdpCycle_HANKPartial = gdpCycle_HANK(12:end);
cCycle_HANK = regcyc(cumsum(cGrowth_HANK)); 
cCycle_HANKPartial = cCycle_HANK(12:end);
iCycle_HANK = regcyc(cumsum(iGrowth_HANK)); 
iCycle_HANKPartial = iCycle_HANK(12:end);
findingRate_HANKPartial = outcome(:,strmatch('obsF', M_.endo_names, 'exact'));
lossRate_HANKPartial = outcome(:,strmatch('obsS', M_.endo_names, 'exact'));

gdpGrowth_RANK = meanGDPGrowth + outcome(:,strmatch('obsgX_R', M_.endo_names, 'exact'));
cGrowth_RANK = meanCGrowth + outcome(:,strmatch('obsgC_R', M_.endo_names, 'exact'));
iGrowth_RANK = meanIGrowth + outcome(:,strmatch('obsgI_R', M_.endo_names, 'exact'));
gdpCycle_RANK = regcyc(cumsum(gdpGrowth_RANK)); 
gdpCycle_RANKPartial = gdpCycle_RANK(12:end);
cCycle_RANK = regcyc(cumsum(cGrowth_RANK)); 
cCycle_RANKPartial = cCycle_RANK(12:end);
iCycle_RANK = regcyc(cumsum(iGrowth_RANK)); 
iCycle_RANKPartial = iCycle_RANK(12:end);
findingRate_RANKPartial = outcome(:,strmatch('obsF_R', M_.endo_names, 'exact'));
lossRate_RANKPartial = outcome(:,strmatch('obsS_R', M_.endo_names, 'exact'));

gdpGrowth_TANK = meanGDPGrowth + outcome(:,strmatch('obsgX_T', M_.endo_names, 'exact'));
cGrowth_TANK = meanCGrowth + outcome(:,strmatch('obsgC_T', M_.endo_names, 'exact'));
iGrowth_TANK = meanIGrowth + outcome(:,strmatch('obsgI_T', M_.endo_names, 'exact'));
gdpCycle_TANK = regcyc(cumsum(gdpGrowth_TANK)); 
gdpCycle_TANKPartial = gdpCycle_TANK(12:end);
cCycle_TANK = regcyc(cumsum(cGrowth_TANK)); 
cCycle_TANKPartial = cCycle_TANK(12:end);
iCycle_TANK = regcyc(cumsum(iGrowth_TANK)); 
iCycle_TANKPartial = iCycle_TANK(12:end);
findingRate_TANKPartial = outcome(:,strmatch('obsF_T', M_.endo_names, 'exact'));
lossRate_TANKPartial = outcome(:,strmatch('obsS_T', M_.endo_names, 'exact'));

%Feed the smoothed shocks into the constant risk model and generate the model-implied time-series  
dynare counterFactuals.mod -Dmodel_option=2 -DnMeasure_option=4

gdpGrowth_CRisk = meanGDPGrowth + outcome(:,strmatch('obsgX', M_.endo_names, 'exact'));
cGrowth_CRisk = meanCGrowth + outcome(:,strmatch('obsgC', M_.endo_names, 'exact'));
iGrowth_CRisk = meanIGrowth + outcome(:,strmatch('obsgI', M_.endo_names, 'exact'));
gdpCycle_CRisk = regcyc(cumsum(gdpGrowth_CRisk)); 
gdpCycle_CRiskPartial = gdpCycle_CRisk(12:end);
cCycle_CRisk = regcyc(cumsum(cGrowth_CRisk)); 
cCycle_CRiskPartial = cCycle_CRisk(12:end);
iCycle_CRisk = regcyc(cumsum(iGrowth_CRisk)); 
iCycle_CRiskPartial = iCycle_CRisk(12:end);
findingRate_CRiskPartial = outcome(:,strmatch('obsF', M_.endo_names, 'exact'));
lossRate_CRiskPartial = outcome(:,strmatch('obsS', M_.endo_names, 'exact'));  

%Volatility of the model-implied time-series (Table 4)   
stdTablePartial = [std(gdpCycle_HANKPartial) std(cCycle_HANKPartial) std(iCycle_HANKPartial) std(findingRate_HANKPartial) std(lossRate_HANKPartial);
                  std(gdpCycle_CRiskPartial) std(cCycle_CRiskPartial) std(iCycle_CRiskPartial) std(findingRate_CRiskPartial) std(lossRate_CRiskPartial);
                  std(gdpCycle_RANKPartial) std(cCycle_RANKPartial) std(iCycle_RANKPartial) std(findingRate_RANKPartial) std(lossRate_RANKPartial)];
                 


delete *_dynamic.m
delete *_static.m
delete *.log
delete *_set_auxiliary_variables.m
delete *_results.mat
delete *.eps
delete *.asv


% Figure plots 
load RecShading_1967q1_2010q1


% Dates
DateLabels.PlotStart = '1967q1';
DateLabels.PlotEnd = '2010q1';
DateLabels.XTick = [13 53 93 133 173];
DateLabels.XTickLabels = {'1970','1980','1990','2000','2010'};
T= [1967.25:0.25:2008.75];
tid=[1:1:length(T)]';

% HANK vs HANK (const. risk) vs RANK subject to partial shocks (Figure 2)
FigHandle = figure('Position', [150, 150, 1200, 800]);
set(0,'defaultlinelinewidth',1.5)

PlotValues = [zeros(length(T),1); gdpCycle_HANKPartial; gdpCycle_CRiskPartial; gdpCycle_RANKPartial];
yBounds1 = [min(PlotValues(:)),max(PlotValues(:))];

subaxis(2,2,1:2,'spacing',0.1);
bar(yBounds1(2)*RecShading,1,'EdgeColor','none','FaceColor',.85*[1,1,1])
hold on
bar(yBounds1(1)*RecShading,1,'EdgeColor','none','FaceColor',.85*[1,1,1])
plot(tid,gdpCycle_HANKPartial,'-', tid, gdpCycle_CRiskPartial,'k-.', tid,gdpCycle_RANKPartial,'--');
hold off
ylim(yBounds1)
xlim([1 173]);
set(gca,'XTick',DateLabels.XTick,'XTickLabel',DateLabels.XTickLabels,'FontSize',15)
title({'GDP'},'Interpreter','latex'); grid on; ylabel({'Proportional deviation (\%)'},'Interpreter','latex','FontSize',13);

PlotValues = [zeros(length(T),1); cCycle_HANKPartial; cCycle_CRiskPartial; cCycle_RANKPartial];
yBounds1 = [min(PlotValues(:)),max(PlotValues(:))];

subaxis(2,2,3:4);
bar(yBounds1(2)*RecShading,1,'EdgeColor','none','FaceColor',.85*[1,1,1])
hold on
bar(yBounds1(1)*RecShading,1,'EdgeColor','none','FaceColor',.85*[1,1,1])
plot(tid,cCycle_HANKPartial,'-', tid, cCycle_CRiskPartial,'k-.', tid,cCycle_RANKPartial,'--');
hold off
ylim(yBounds1)
xlim([1 173]);
set(gca,'XTick',DateLabels.XTick,'XTickLabel',DateLabels.XTickLabels,'FontSize',15)
title({'Consumption'},'Interpreter','latex'); grid on; ylabel({'Proportional deviation (\%)'},'Interpreter','latex','FontSize',13);
set(gcf,'color','w');


FigHandle = figure('Position', [150, 150, 1200, 400]);
set(0,'defaultlinelinewidth',1.5)

PlotValues = [zeros(length(T),1); findingRate_HANKPartial; findingRate_CRiskPartial; findingRate_RANKPartial];
yBounds1 = [min(PlotValues(:)),max(PlotValues(:))];

subaxis(1,2,1,'spacing',0.1);
plot(tid,findingRate_HANKPartial(12:end),'-',tid,findingRate_CRiskPartial(12:end),'k-.',tid,findingRate_RANKPartial(12:end),'--');
xlim([1 173]);
ylim(yBounds1)
set(gca,'XTick',DateLabels.XTick,'XTickLabel',DateLabels.XTickLabels,'FontSize',15)
title({'Job-finding rate'},'Interpreter','latex'); grid on; ylabel({'Level deviation (\%)'},'Interpreter','latex','FontSize',13);

subaxis(1,2,2);
plot(tid,lossRate_HANKPartial(12:end),'-',tid,lossRate_CRiskPartial(12:end),'k-.',tid,lossRate_RANKPartial(12:end),'--');
set(gca,'XTick',DateLabels.XTick,'XTickLabel',DateLabels.XTickLabels,'FontSize',15)
title({'Job-loss rate'},'Interpreter','latex'); grid on; ylabel({'Level deviation (\%)'},'Interpreter','latex','FontSize',13);
xlim([1 173]);
ylim(yBounds1)

lg = legend({'HANK','HANK (const. risk)','RANK'},'Interpreter','latex','location','best'); 
set(lg,'FontSize',15);
legend('boxoff');
set(gcf,'color','w');

% HANK vs TANK (const. risk) vs TANK subject to partial shocks (Figure 6)
FigHandle = figure('Position', [150, 150, 1200, 800]);
set(0,'defaultlinelinewidth',1.5)

PlotValues = [zeros(length(T),1); gdpCycle_HANKPartial; gdpCycle_CRiskPartial; gdpCycle_TANKPartial];
yBounds1 = [min(PlotValues(:)),max(PlotValues(:))];

subaxis(2,2,1:2,'spacing',0.1);
bar(yBounds1(2)*RecShading,1,'EdgeColor','none','FaceColor',.85*[1,1,1])
hold on
bar(yBounds1(1)*RecShading,1,'EdgeColor','none','FaceColor',.85*[1,1,1])
plot(tid,gdpCycle_HANKPartial,'-', tid, gdpCycle_CRiskPartial,'k-.', tid,gdpCycle_TANKPartial,'m--');
hold off
ylim(yBounds1)
xlim([1 173]);
set(gca,'XTick',DateLabels.XTick,'XTickLabel',DateLabels.XTickLabels,'FontSize',15)
title({'GDP'},'Interpreter','latex'); grid on; ylabel({'Proportional deviation (\%)'},'Interpreter','latex','FontSize',13);

PlotValues = [zeros(length(T),1); cCycle_HANKPartial; cCycle_CRiskPartial; cCycle_TANKPartial];
yBounds1 = [min(PlotValues(:)),max(PlotValues(:))];

subaxis(2,2,3:4);
bar(yBounds1(2)*RecShading,1,'EdgeColor','none','FaceColor',.85*[1,1,1])
hold on
bar(yBounds1(1)*RecShading,1,'EdgeColor','none','FaceColor',.85*[1,1,1])
p = plot(tid,cCycle_HANKPartial,'-', tid, cCycle_CRiskPartial,'k-.', tid,cCycle_TANKPartial,'m--');
hold off
ylim(yBounds1)
xlim([1 173]);
set(gca,'XTick',DateLabels.XTick,'XTickLabel',DateLabels.XTickLabels,'FontSize',15)
title({'Consumption'},'Interpreter','latex'); grid on; ylabel({'Proportional deviation (\%)'},'Interpreter','latex','FontSize',13);
set(gcf,'color','w');

lg =  legend([p],{'HANK','HANK (const. risk)','TANK'},'Interpreter','latex','location','best'); 
set(lg,'FontSize',13);
legend('boxoff');




%------------------------------------------------------------------------
% Compute moments of aggregate variables subject to all shocks   
% Produces Figure A.2 and A.4
%------------------------------------------------------------------------
load('smoothedShocks')

% HANK 
dynare counterFactuals.mod -Dmodel_option=1 -DnMeasure_option=4

gdpGrowth_HANK = meanGDPGrowth + outcome(:,strmatch('obsgX', M_.endo_names, 'exact'));
cGrowth_HANK = meanCGrowth + outcome(:,strmatch('obsgC', M_.endo_names, 'exact'));
iGrowth_HANK = meanIGrowth + outcome(:,strmatch('obsgI', M_.endo_names, 'exact'));
gdpCycle_HANK = regcyc(cumsum(gdpGrowth_HANK)); 
gdpCycle_HANKAll = gdpCycle_HANK(12:end);
cCycle_HANK = regcyc(cumsum(cGrowth_HANK)); 
cCycle_HANKAll = cCycle_HANK(12:end);
iCycle_HANK = regcyc(cumsum(iGrowth_HANK)); 
iCycle_HANKAll = iCycle_HANK(12:end);
findingRate_HANKAll = outcome(:,strmatch('obsF', M_.endo_names, 'exact'));
lossRate_HANKAll = outcome(:,strmatch('obsS', M_.endo_names, 'exact'));
uRate_HANKAll = outcome(:,strmatch('obsU', M_.endo_names, 'exact'));
vacancy_HANKAll = outcome(:,strmatch('vHat', M_.endo_names, 'exact'));


gdpGrowth_RANK = meanGDPGrowth + outcome(:,strmatch('obsgX_R', M_.endo_names, 'exact'));
cGrowth_RANK = meanCGrowth + outcome(:,strmatch('obsgC_R', M_.endo_names, 'exact'));
iGrowth_RANK = meanIGrowth + outcome(:,strmatch('obsgI_R', M_.endo_names, 'exact'));
gdpCycle_RANK = regcyc(cumsum(gdpGrowth_RANK)); 
gdpCycle_RANKAll = gdpCycle_RANK(12:end);
cCycle_RANK = regcyc(cumsum(cGrowth_RANK)); 
cCycle_RANKAll = cCycle_RANK(12:end);
iCycle_RANK = regcyc(cumsum(iGrowth_RANK)); 
iCycle_RANKAll = iCycle_RANK(12:end);
findingRate_RANKAll = outcome(:,strmatch('obsF_R', M_.endo_names, 'exact'));
lossRate_RANKAll = outcome(:,strmatch('obsS_R', M_.endo_names, 'exact'));
uRate_RANKAll = outcome(:,strmatch('obsU_R', M_.endo_names, 'exact'));



% Constant risk
dynare counterFactuals.mod -Dmodel_option=2 -DnMeasure_option=4

gdpGrowth_CRisk = meanGDPGrowth + outcome(:,strmatch('obsgX', M_.endo_names, 'exact'));
cGrowth_CRisk = meanCGrowth + outcome(:,strmatch('obsgC', M_.endo_names, 'exact'));
iGrowth_CRisk = meanIGrowth + outcome(:,strmatch('obsgI', M_.endo_names, 'exact'));
gdpCycle_CRisk = regcyc(cumsum(gdpGrowth_CRisk)); 
gdpCycle_CRiskAll = gdpCycle_CRisk(12:end);
cCycle_CRisk = regcyc(cumsum(cGrowth_CRisk)); 
cCycle_CRiskAll = cCycle_CRisk(12:end);
iCycle_CRisk = regcyc(cumsum(iGrowth_CRisk)); 
iCycle_CRiskAll = iCycle_CRisk(12:end);
findingRate_CRiskAll = outcome(:,strmatch('obsF', M_.endo_names, 'exact'));
lossRate_CRiskAll = outcome(:,strmatch('obsS', M_.endo_names, 'exact'));
uRate_CRiskAll = outcome(:,strmatch('obsU', M_.endo_names, 'exact'));


% Figure plots 
load RecShading_1967q1_2010q1


% Dates
DateLabels.PlotStart = '1967q1';
DateLabels.PlotEnd = '2010q1';
DateLabels.XTick = [13 53 93 133 173];
DateLabels.XTickLabels = {'1970','1980','1990','2000','2010'};
T= [1967.25:0.25:2008.75];
tid=[1:1:length(T)]';

% HANK vs Data subject to all shocks for labor market variables (Figure A.2) 
load('estimationData')

FigHandle = figure('Position', [150, 150, 1200, 900]);
set(0,'defaultlinelinewidth',1.5)

subaxis(2,2,1,'spacing',0.1);
plot(tid,findingRate_HANKAll(12:end),'-',tid,obsF(12:end),'r--');
xlim([1 173]);
set(gca,'XTick',DateLabels.XTick,'XTickLabel',DateLabels.XTickLabels,'FontSize',15)
title({'Job-finding rate'},'Interpreter','latex'); grid on; ylabel({'Level deviation (\%)'},'Interpreter','latex','FontSize',13);

subaxis(2,2,2,'spacing',0.1);
plot(tid,lossRate_HANKAll(12:end),'-',tid,obsS(12:end),'r--');
set(gca,'XTick',DateLabels.XTick,'XTickLabel',DateLabels.XTickLabels,'FontSize',15)
title({'Job-loss rate'},'Interpreter','latex'); grid on; ylabel({'Level deviation (\%)'},'Interpreter','latex','FontSize',13);
xlim([1 173]);
ylim([-30 20])

subaxis(2,2,3,'spacing',0.1);
plot(tid,uRate_HANKAll(12:end),'-',tid,obsU(12:end),'r--');
set(gca,'XTick',DateLabels.XTick,'XTickLabel',DateLabels.XTickLabels,'FontSize',15)
title({'Unemployment rate'},'Interpreter','latex'); grid on; ylabel({'Level deviation (\%)'},'Interpreter','latex','FontSize',13);
xlim([1 173]);
lg = legend({'HANK','Data'},'Interpreter','latex','location','best'); 
set(lg,'FontSize',15);
legend('boxoff');

subaxis(2,2,4);
plot(tid,vacancy_HANKAll(12:end),'-',tid,obsV(12:end),'r--');
set(gca,'XTick',DateLabels.XTick,'XTickLabel',DateLabels.XTickLabels,'FontSize',15)
title({'Vacancy'},'Interpreter','latex'); grid on; ylabel({'Proportional deviation (\%)'},'Interpreter','latex','FontSize',13);
xlim([1 173]);
set(gcf,'color','w');


% HANK vs HANK (const. risk) vs RANK subject to all shocks (Figure A.4)
FigHandle = figure('Position', [150, 150, 1200, 800]);
set(0,'defaultlinelinewidth',1.5)

PlotValues = [zeros(length(T),1); gdpCycle_HANKAll; gdpCycle_CRiskAll; gdpCycle_RANKAll];
yBounds1 = [min(PlotValues(:)),max(PlotValues(:))];

subaxis(2,2,1:2,'spacing',0.11);
bar(yBounds1(2)*RecShading,1,'EdgeColor','none','FaceColor',.85*[1,1,1])
hold on
bar(yBounds1(1)*RecShading,1,'EdgeColor','none','FaceColor',.85*[1,1,1])
plot(tid,gdpCycle_HANKAll,'-', tid, gdpCycle_CRiskAll,'k-.', tid,gdpCycle_RANKAll,'--');
hold off
ylim(yBounds1)
xlim([1 173]);
set(gca,'XTick',DateLabels.XTick,'XTickLabel',DateLabels.XTickLabels,'FontSize',15)
title({'GDP'},'Interpreter','latex'); grid on; ylabel({'Proportional deviation (\%)'},'Interpreter','latex','FontSize',13);

PlotValues = [zeros(length(T),1); cCycle_HANKAll; cCycle_CRiskAll; cCycle_RANKAll];
yBounds1 = [min(PlotValues(:)),max(PlotValues(:))];

subaxis(2,2,3:4);
bar(yBounds1(2)*RecShading,1,'EdgeColor','none','FaceColor',.85*[1,1,1])
hold on
bar(yBounds1(1)*RecShading,1,'EdgeColor','none','FaceColor',.85*[1,1,1])
plot(tid,cCycle_HANKAll,'-', tid, cCycle_CRiskAll,'k-.', tid,cCycle_RANKAll,'--');
hold off
ylim(yBounds1)
xlim([1 173]);
set(gca,'XTick',DateLabels.XTick,'XTickLabel',DateLabels.XTickLabels,'FontSize',15)
title({'Consumption'},'Interpreter','latex'); grid on; ylabel({'Proportional deviation (\%)'},'Interpreter','latex','FontSize',13);
set(gcf,'color','w');


FigHandle = figure('Position', [150, 150, 1200, 400]);
set(0,'defaultlinelinewidth',1.5)

subaxis(1,2,1,'spacing',0.1);
plot(tid,findingRate_HANKAll(12:end),'-',tid,findingRate_CRiskAll(12:end),'k-.',tid,findingRate_RANKAll(12:end),'--');
xlim([1 173]);
set(gca,'XTick',DateLabels.XTick,'XTickLabel',DateLabels.XTickLabels,'FontSize',15)
title({'Job-finding rate'},'Interpreter','latex'); grid on; ylabel({'Level deviation (\%)'},'Interpreter','latex','FontSize',13);

subaxis(1,2,2);
plot(tid,lossRate_HANKAll(12:end),'-',tid,lossRate_CRiskAll(12:end),'k-.',tid,lossRate_RANKAll(12:end),'--');
set(gca,'XTick',DateLabels.XTick,'XTickLabel',DateLabels.XTickLabels,'FontSize',15)
title({'Job-loss rate'},'Interpreter','latex'); grid on; ylabel({'Level deviation (\%)'},'Interpreter','latex','FontSize',13);
xlim([1 173]);
ylim([-30 20])

lg = legend({'HANK','HANK (const. risk)','RANK'},'Interpreter','latex','location','best'); 
set(lg,'FontSize',15);
legend('boxoff');
set(gcf,'color','w');




%------------------------------------------------------------------------
% IRFs   
% Produces Figures 4, A.3, A.5, A.6, and A.7
%------------------------------------------------------------------------

% IRF to preference shock (Figure A.3)

load('HANKparam')
setParameters;

save economicParameters.mat bbetaL ssigma aaBar bbetaH Hshare Hshare_T aalpha ddelta ss2prime rrhouk ppsi ...
     xxiP iiotaP ggamma rrhoxBar kkappa MMBar bb iiotaW xxiW FF rrhoR pphiPi pphiX vEpsilonGrid ...
     rrhoetaP rrhomuu rrhoupsilon rrhoTFP rrhozeta rrhorhox rrhopiStar rrhog... 
     ssigmaetaP ssigmamuu ssigmaupsilon ssigmaTFP ssigmaR ssigmazeta ssigmarhox...
     ssigmapiStar ssigmag ssigmaW;

% HANK
dynare HANKDynamicsShocks.mod -Dmodel_option=1 -DnMeasure_option=4
preferenceIRF_HANK = oo_.irfs; 

% HANK with constant risk
dynare HANKDynamicsShocks.mod -Dmodel_option=2 -DnMeasure_option=4
preferenceIRF_Crisk = oo_.irfs;  

varlist_HANK = {'cHat','invHat','xHat','nHat','obsF','obsS'}; 
varlist_RANK = {'cHat_R','invHat_R','xHat_R','nHat_R','obsF_R','obsS_R'};

titlelist = {'Consumption','Investment','GDP','Employment rate','Job-finding rate','Job-loss rate'};
        
   
filelist = {preferenceIRF_HANK, preferenceIRF_Crisk};    

FigHandle = figure('Position', [150, 150, 1200, 800]);
set(0,'defaultlinelinewidth',2.5)

time = [0:19]';

for iplot = 1:length(titlelist)
    subaxis(2,3,iplot,'spacing',0.1), plot(time,getfield(filelist{1},strjoin(strcat(varlist_HANK(iplot),'_epszeta'))),'-',...
                    time,getfield(filelist{2},strjoin(strcat(varlist_HANK(iplot),'_epszeta'))), 'k-.',...
                    time,getfield(filelist{1},strjoin(strcat(varlist_RANK(iplot),'_epszeta'))), '--' ); 
    title(titlelist(iplot),'FontSize',15), grid on;  
    if iplot < 4
        ylabel({'Proportional deviation (\%)'},'Interpreter','latex','FontSize',13);
    else 
        ylabel({'Level deviation (\%)'},'Interpreter','latex','FontSize',13)
    end
end
lg = legend({'HANK','HANK (const. risk)','RANK'},'Interpreter','latex','location','best'); grid on;
set(lg,'FontSize',15);
legend('boxoff');
lg.Orientation = 'horizontal';
set(lg,'position',[0,0,1,0.08]);
set(gcf,'color','w');



% IRF to MEI shock plotting (Figure 4)
load('HANKparam')
setParameters;

save economicParameters.mat bbetaL ssigma aaBar bbetaH Hshare Hshare_T aalpha ddelta ss2prime rrhouk ppsi ...
     xxiP iiotaP ggamma rrhoxBar kkappa MMBar bb iiotaW xxiW FF rrhoR pphiPi pphiX vEpsilonGrid ...
     rrhoetaP rrhomuu rrhoupsilon rrhoTFP rrhozeta rrhorhox rrhopiStar rrhog... 
     ssigmaetaP ssigmamuu ssigmaupsilon ssigmaTFP ssigmaR ssigmazeta ssigmarhox...
     ssigmapiStar ssigmag ssigmaW;
 
% HANK
dynare HANKDynamicsShocks.mod -Dmodel_option=1 -DnMeasure_option=4       
IRF_HANK = oo_.irfs;    

% HANK with constant risk
dynare HANKDynamicsShocks.mod -Dmodel_option=2 -DnMeasure_option=4
IRF_Crisk = oo_.irfs;  


load('RANKparam')
setParameters;

save economicParameters.mat bbetaL ssigma aaBar bbetaH Hshare Hshare_T aalpha ddelta ss2prime rrhouk ppsi ...
     xxiP iiotaP ggamma rrhoxBar kkappa MMBar bb iiotaW xxiW FF rrhoR pphiPi pphiX vEpsilonGrid ...
     rrhoetaP rrhomuu rrhoupsilon rrhoTFP rrhozeta rrhorhox rrhopiStar rrhog... 
     ssigmaetaP ssigmamuu ssigmaupsilon ssigmaTFP ssigmaR ssigmazeta ssigmarhox...
     ssigmapiStar ssigmag ssigmaW;
 
dynare RANKDynamics
IRF_RANK = oo_.irfs;  

filelist = {IRF_HANK, IRF_Crisk, IRF_RANK};    


time = [0:19]';

varlist = {'cHat','invHat','xHat','nHat','obsF','obsS'}; 

titlelist = {'Consumption','Investment','GDP','Employment rate','Job-finding rate','Job-loss rate'};


FigHandle = figure('Position', [150, 150, 1200, 800]);
set(0,'defaultlinelinewidth',2.5)
for iplot = 1:length(titlelist)
    subaxis(2,3,iplot,'spacing',0.1), plot(time,getfield(filelist{1},strjoin(strcat(varlist(iplot),'_epsupsilon'))),'-',...
                    time,getfield(filelist{2},strjoin(strcat(varlist(iplot),'_epsupsilon'))), 'k-.',...
                    time,getfield(filelist{3},strjoin(strcat(varlist(iplot),'_epsupsilon'))), '--' ); 
    title(titlelist(iplot),'FontSize',15), grid on;  
    if iplot < 4
        ylabel({'Proportional deviation (\%)'},'Interpreter','latex','FontSize',13);
    else 
        ylabel({'Level deviation (\%)'},'Interpreter','latex','FontSize',13);
    end
end
lg = legend({'HANK','HANK (const. risk)','RANK'},'Interpreter','latex','location','best'); grid on;
set(lg,'FontSize',15);
legend('boxoff');
lg.Orientation = 'horizontal';
set(lg,'position',[0,0,1,0.08]);
set(gcf,'color','w');


% IRF to a markup shock (Figure A.5(a))

varlist = {'cHat','invHat','xHat','nHat','obsF','obsS','obsPi','obsFF'}; 

titlelist = {'Consumption','Investment','GDP','Employment rate','Job-finding rate','Job-loss rate', 'Inflation','Interest rate'};

FigHandle = figure('Position', [150, 150, 1200, 450]);
set(0,'defaultlinelinewidth',2.5)

for iplot = 1:length(titlelist)
    subaxis(2,4,iplot,'spacing',0.1), plot(time,getfield(filelist{1},strjoin(strcat(varlist(iplot),'_epsetaP'))),'-',...
                    time,getfield(filelist{3},strjoin(strcat(varlist(iplot),'_epsetaP'))), '--' ); 
    title(titlelist(iplot),'FontSize',15), grid on; 
    if iplot < 5
        ylabel({'Prop. deviation (\%)'},'Interpreter','latex','FontSize',12);
    else 
        ylabel({'Level deviation (\%)'},'Interpreter','latex','FontSize',12);
    end
end
lg = legend({'HANK','RANK'},'Interpreter','latex','location','best'); grid on;
set(lg,'FontSize',15);
legend('boxoff');
lg.Orientation = 'horizontal';
set(lg,'position',[0,0,1,0.08]);
set(gcf,'color','w');


% IRF to a TFP growth shock (Figure A.5(b))
shocklist = {'epsetaP', 'epsmuu', 'epsupsilon', 'epsTFP', 'epsR',...
            'epszeta', 'epsrhox','epsg', 'epsW'};
varlistg = {'obsgC','obsgI','obsgX','obsgG'}; 

titlelist = {'Consumption','Investment','GDP','Employment rate','Job-finding rate','Job-loss rate', 'Inflation','Interest rate'};

% Transforming variables that have trend to gross growth rates with positive mean
% The rest of the variables remain untouched

muuSS = 1.0047;
growthrates_IRF_HANK = IRF_HANK;
for ishock = 1:length(shocklist)
    for ivar = 1:length(varlistg)
        field1 = strjoin(strcat(varlistg(ivar),'_',shocklist(ishock)));
        value = getfield(growthrates_IRF_HANK,strjoin(strcat(varlistg(ivar),'_',shocklist(ishock))));   
        obsg = exp(value + log(muuSS));
        growthrates_IRF_HANK = setfield(growthrates_IRF_HANK,field1,obsg');
    end
end

growthrates_IRF_Crisk = IRF_Crisk;
for ishock = 1:length(shocklist)
    for ivar = 1:length(varlistg)
        field1 = strjoin(strcat(varlistg(ivar),'_',shocklist(ishock)));
        value = getfield(growthrates_IRF_Crisk,strjoin(strcat(varlistg(ivar),'_',shocklist(ishock))));   
        obsg = exp(value + log(muuSS));
        growthrates_IRF_Crisk = setfield(growthrates_IRF_Crisk,field1,obsg');
    end
end

growthrates_IRF_RANK = IRF_RANK;
for ishock = 1:length(shocklist)
    for ivar = 1:length(varlistg)
        field1 = strjoin(strcat(varlistg(ivar),'_',shocklist(ishock)));
        value = getfield(growthrates_IRF_RANK,strjoin(strcat(varlistg(ivar),'_',shocklist(ishock))));   
        obsg = exp(value + log(muuSS));
        growthrates_IRF_RANK = setfield(growthrates_IRF_RANK,field1,obsg');
    end
end

% Transforming growth rate variables to percent deviation from deterministic trend
% The rest of the variables remain untouched

Ctrend = ones(20,1)*muuSS;

percentdev_IRF_HANK = growthrates_IRF_HANK;
for ishock = 1:length(shocklist)
    for ivar = 1:length(varlistg)
    field1 = strjoin(strcat(varlistg(ivar),'_',shocklist(ishock)));
    value = getfield(percentdev_IRF_HANK,strjoin(strcat(varlistg(ivar),'_',shocklist(ishock))));   
    percentdev = (cumprod(value) - cumprod(Ctrend))./cumprod(Ctrend);
        
    percentdev_IRF_HANK = setfield(percentdev_IRF_HANK,field1,percentdev);
    end
end

percentdev_IRF_Crisk = growthrates_IRF_Crisk;
for ishock = 1:length(shocklist)
    for ivar = 1:length(varlistg)
    field1 = strjoin(strcat(varlistg(ivar),'_',shocklist(ishock)));
    value = getfield(percentdev_IRF_Crisk,strjoin(strcat(varlistg(ivar),'_',shocklist(ishock))));   
    percentdev = (cumprod(value) - cumprod(Ctrend))./cumprod(Ctrend);
        
    percentdev_IRF_Crisk = setfield(percentdev_IRF_Crisk,field1,percentdev);
    end
end

percentdev_IRF_RANK = growthrates_IRF_RANK;
for ishock = 1:length(shocklist)
    for ivar = 1:length(varlistg)
    field1 = strjoin(strcat(varlistg(ivar),'_',shocklist(ishock)));
    value = getfield(percentdev_IRF_RANK,strjoin(strcat(varlistg(ivar),'_',shocklist(ishock))));   
    percentdev = (cumprod(value) - cumprod(Ctrend))./cumprod(Ctrend);
        
    percentdev_IRF_RANK = setfield(percentdev_IRF_RANK,field1,percentdev);
    end
end

filelist1 = {percentdev_IRF_HANK, percentdev_IRF_Crisk, percentdev_IRF_RANK};    
varlist = {'obsgC','obsgI','obsgX','nHat','obsF','obsS','obsPi','obsFF'}; 

FigHandle = figure('Position', [150, 150, 1200, 450]);
set(0,'defaultlinelinewidth',2.5)

for iplot = 1:length(titlelist)
    subaxis(2,4,iplot,'spacing',0.1), plot(time,getfield(filelist1{1},strjoin(strcat(varlist(iplot),'_epsmuu'))),'-',...
                    time,getfield(filelist1{3},strjoin(strcat(varlist(iplot),'_epsmuu'))), '--' ); 
    title(titlelist(iplot),'FontSize',15), grid on; 
    if iplot < 5
        ylabel({'Prop. deviation (\%)'},'Interpreter','latex','FontSize',12);
    else 
        ylabel({'Level deviation (\%)'},'Interpreter','latex','FontSize',12);
    end
end
lg = legend({'HANK','RANK'},'Interpreter','latex','location','best'); grid on;
set(lg,'FontSize',15);
legend('boxoff');
lg.Orientation = 'horizontal';
set(lg,'position',[0,0,1,0.08]);
set(gcf,'color','w');


% IRF to a MEI shock (Figure A.5(c))
varlist = {'cHat','invHat','xHat','nHat','obsF','obsS','obsPi','obsFF'}; 

titlelist = {'Consumption','Investment','GDP','Employment rate','Job-finding rate','Job-loss rate', 'Inflation','Interest rate'};

FigHandle = figure('Position', [150, 150, 1200, 450]);
set(0,'defaultlinelinewidth',2.5)

for iplot = 1:length(titlelist)
    subaxis(2,4,iplot,'spacing',0.1), plot(time,getfield(filelist{1},strjoin(strcat(varlist(iplot),'_epsupsilon'))),'-',...
                    time,getfield(filelist{3},strjoin(strcat(varlist(iplot),'_epsupsilon'))), '--' ); 
    title(titlelist(iplot),'FontSize',15), grid on; 
    if iplot < 5
        ylabel({'Prop. deviation (\%)'},'Interpreter','latex','FontSize',12);
    else 
        ylabel({'Level deviation (\%)'},'Interpreter','latex','FontSize',12);
    end
end
lg = legend({'HANK','RANK'},'Interpreter','latex','location','best'); grid on;
set(lg,'FontSize',15);
legend('boxoff');
lg.Orientation = 'horizontal';
set(lg,'position',[0,0,1,0.08]);
set(gcf,'color','w');


% IRF to a TFP shock (Figure A.6(a))
varlist = {'cHat','invHat','xHat','nHat','obsF','obsS','obsPi','obsFF'}; 

titlelist = {'Consumption','Investment','GDP','Employment rate','Job-finding rate','Job-loss rate', 'Inflation','Interest rate'};

FigHandle = figure('Position', [150, 150, 1200, 450]);
set(0,'defaultlinelinewidth',2.5)

for iplot = 1:length(titlelist)
    subaxis(2,4,iplot,'spacing',0.1), plot(time,getfield(filelist{1},strjoin(strcat(varlist(iplot),'_epsTFP'))),'-',...
                    time,getfield(filelist{3},strjoin(strcat(varlist(iplot),'_epsTFP'))), '--' ); 
    title(titlelist(iplot),'FontSize',15), grid on; 
    if iplot < 5
        ylabel({'Prop. deviation (\%)'},'Interpreter','latex','FontSize',12);
    else 
        ylabel({'Level deviation (\%)'},'Interpreter','latex','FontSize',12);
    end
end
lg = legend({'HANK','RANK'},'Interpreter','latex','location','best'); grid on;
set(lg,'FontSize',15);
legend('boxoff');
lg.Orientation = 'horizontal';
set(lg,'position',[0,0,1,0.08]);
set(gcf,'color','w');


% IRF to a MP shock (Figure A.6(b))
varlist = {'cHat','invHat','xHat','nHat','obsF','obsS','obsPi','obsFF'}; 

titlelist = {'Consumption','Investment','GDP','Employment rate','Job-finding rate','Job-loss rate', 'Inflation','Interest rate'};

FigHandle = figure('Position', [150, 150, 1200, 450]);
set(0,'defaultlinelinewidth',2.5)

for iplot = 1:length(titlelist)
    subaxis(2,4,iplot,'spacing',0.1), plot(time,getfield(filelist{1},strjoin(strcat(varlist(iplot),'_epsR'))),'-',...
                    time,getfield(filelist{3},strjoin(strcat(varlist(iplot),'_epsR'))), '--' ); 
    title(titlelist(iplot),'FontSize',15), grid on; 
    if iplot < 5
        ylabel({'Prop. deviation (\%)'},'Interpreter','latex','FontSize',12);
    else 
        ylabel({'Level deviation (\%)'},'Interpreter','latex','FontSize',12);
    end
end
lg = legend({'HANK','RANK'},'Interpreter','latex','location','best'); grid on;
set(lg,'FontSize',15);
legend('boxoff');
lg.Orientation = 'horizontal';
set(lg,'position',[0,0,1,0.08]);
set(gcf,'color','w');


% IRF to a preference shock (Figure A.6(c))
varlist = {'cHat','invHat','xHat','nHat','obsF','obsS','obsPi','obsFF'}; 

titlelist = {'Consumption','Investment','GDP','Employment rate','Job-finding rate','Job-loss rate', 'Inflation','Interest rate'};

FigHandle = figure('Position', [150, 150, 1200, 450]);
set(0,'defaultlinelinewidth',2.5)

for iplot = 1:length(titlelist)
    subaxis(2,4,iplot,'spacing',0.1), plot(time,getfield(filelist{1},strjoin(strcat(varlist(iplot),'_epszeta'))),'-',...
                    time,getfield(filelist{3},strjoin(strcat(varlist(iplot),'_epszeta'))), '--' ); 
    title(titlelist(iplot),'FontSize',15), grid on; 
    if iplot < 5
        ylabel({'Prop. deviation (\%)'},'Interpreter','latex','FontSize',12);
    else 
        ylabel({'Level deviation (\%)'},'Interpreter','latex','FontSize',12);
    end
end
lg = legend({'HANK','RANK'},'Interpreter','latex','location','best'); grid on;
set(lg,'FontSize',15);
legend('boxoff');
lg.Orientation = 'horizontal';
set(lg,'position',[0,0,1,0.08]);
set(gcf,'color','w');


% IRF to a jop separation shock (Figure A.7(a))
varlist = {'cHat','invHat','xHat','nHat','obsF','obsS','obsPi','obsFF'}; 

titlelist = {'Consumption','Investment','GDP','Employment rate','Job-finding rate','Job-loss rate', 'Inflation','Interest rate'};

FigHandle = figure('Position', [150, 150, 1200, 450]);
set(0,'defaultlinelinewidth',2.5)

for iplot = 1:length(titlelist)
    subaxis(2,4,iplot,'spacing',0.1), plot(time,getfield(filelist{1},strjoin(strcat(varlist(iplot),'_epsrhox'))),'-',...
                    time,getfield(filelist{3},strjoin(strcat(varlist(iplot),'_epsrhox'))), '--' ); 
    title(titlelist(iplot),'FontSize',15), grid on; 
    if iplot < 5
        ylabel({'Prop. deviation (\%)'},'Interpreter','latex','FontSize',12);
    else 
        ylabel({'Level deviation (\%)'},'Interpreter','latex','FontSize',12);
    end
end
lg = legend({'HANK','RANK'},'Interpreter','latex','location','best'); grid on;
set(lg,'FontSize',15);
legend('boxoff');
lg.Orientation = 'horizontal';
set(lg,'position',[0,0,1,0.08]);
set(gcf,'color','w');


% IRF to a gov. spending shock (Figure A.7(b))
varlist = {'cHat','invHat','xHat','nHat','obsF','obsS','obsPi','obsFF'}; 

titlelist = {'Consumption','Investment','GDP','Employment rate','Job-finding rate','Job-loss rate', 'Inflation','Interest rate'};

FigHandle = figure('Position', [150, 150, 1200, 450]);
set(0,'defaultlinelinewidth',2.5)

for iplot = 1:length(titlelist)
    subaxis(2,4,iplot,'spacing',0.1), plot(time,getfield(filelist{1},strjoin(strcat(varlist(iplot),'_epsg'))),'-',...
                    time,getfield(filelist{3},strjoin(strcat(varlist(iplot),'_epsg'))), '--' ); 
    title(titlelist(iplot),'FontSize',15), grid on; 
    if iplot < 5
        ylabel({'Prop. deviation (\%)'},'Interpreter','latex','FontSize',12);
    else 
        ylabel({'Level deviation (\%)'},'Interpreter','latex','FontSize',12);
    end
end
lg = legend({'HANK','RANK'},'Interpreter','latex','location','best'); grid on;
set(lg,'FontSize',15);
legend('boxoff');
lg.Orientation = 'horizontal';
set(lg,'position',[0,0,1,0.08]);
set(gcf,'color','w');




%------------------------------------------------------------------------
% Compute moments of aggregate variables subject to all shocks when all
% prices are parameterized as in the RANK model 
% Produces Table 7
%------------------------------------------------------------------------

load('HANKparam')
setParameters;

load('RANKparam')
xxiP = paramMean(12);
iiotaP = paramMean(13);
xxiW = paramMean(14);
iiotaW = paramMean(15);

rrhoR = paramMean(16);
pphiPi = paramMean(17);
pphiX = paramMean(18);

save economicParameters.mat bbetaL ssigma aaBar bbetaH Hshare Hshare_T aalpha ddelta ss2prime rrhouk ppsi ...
     xxiP iiotaP ggamma rrhoxBar kkappa MMBar bb iiotaW xxiW FF rrhoR pphiPi pphiX vEpsilonGrid ...
     rrhoetaP rrhomuu rrhoupsilon rrhoTFP rrhozeta rrhorhox rrhopiStar rrhog... 
     ssigmaetaP ssigmamuu ssigmaupsilon ssigmaTFP ssigmaR ssigmazeta ssigmarhox...
     ssigmapiStar ssigmag ssigmaW;
 
% HANK 
dynare HANKDynamicsShocks.mod -Dmodel_option=1 -DnMeasure_option=4


stdTable_HANKmodelRANKParam = [sqrt(oo_.var(c_pos,c_pos)) sqrt(oo_.var(i_pos,i_pos)) sqrt(oo_.var(pi_pos,pi_pos))...
                    sqrt(oo_.var(f_pos,f_pos)) sqrt(oo_.var(s_pos,s_pos))];
                   
stdTable_T7 = [stdTable_HANKmodel_T7(1,:); stdTable_HANKmodelRANKParam; stdTable_HANKmodel_T7(2,:)]; 

                                      
 
%------------------------------------------------------------------------
% Compute moments of aggregate variables subject to non-preference shocks  
% under alernative calibration
% Produces Tables 5 and 6
%------------------------------------------------------------------------

%--- Less responsive monetary policy rule ---%
load('HANKparam')
setParameters;
pphiPi = 1.2; pphiX = 0; 

save economicParameters.mat bbetaL ssigma aaBar bbetaH Hshare Hshare_T aalpha ddelta ss2prime rrhouk ppsi ...
     xxiP iiotaP ggamma rrhoxBar kkappa MMBar bb iiotaW xxiW FF rrhoR pphiPi pphiX vEpsilonGrid ...
     rrhoetaP rrhomuu rrhoupsilon rrhoTFP rrhozeta rrhorhox rrhopiStar rrhog... 
     ssigmaetaP ssigmamuu ssigmaupsilon ssigmaTFP ssigmaR ssigmazeta ssigmarhox...
     ssigmapiStar ssigmag ssigmaW;
 
shockNames = {'epsetaP', 'epsmuu', 'epsupsilon', 'epsTFP', 'epsR', 'epszeta', 'epsrhox', 'epsg', 'epsW'};

load('smoothedShocks')
load('firstSmoothedValues')

meanCGrowth = 0.53;
meanIGrowth = 0.45;
meanGDPGrowth = 0.47;

smoothedShocks(:,find(strcmp(shockNames, 'epszeta'))) = zeros(size(smoothedShocks,1),1);   
 

% HANK
dynare counterFactuals.mod -Dmodel_option=1 -DnMeasure_option=4

gdpGrowth_HANK = meanGDPGrowth + outcome(:,strmatch('obsgX', M_.endo_names, 'exact'));
cGrowth_HANK = meanCGrowth + outcome(:,strmatch('obsgC', M_.endo_names, 'exact'));
iGrowth_HANK = meanIGrowth + outcome(:,strmatch('obsgI', M_.endo_names, 'exact'));
gdpCycle_HANK = regcyc(cumsum(gdpGrowth_HANK)); 
gdpCycle_HANKPartial_MP = gdpCycle_HANK(12:end);
cCycle_HANK = regcyc(cumsum(cGrowth_HANK)); 
cCycle_HANKPartial_MP = cCycle_HANK(12:end);
iCycle_HANK = regcyc(cumsum(iGrowth_HANK)); 
iCycle_HANKPartial_MP = iCycle_HANK(12:end);
findingRate_HANKPartial_MP = outcome(:,strmatch('obsF', M_.endo_names, 'exact'));
lossRate_HANKPartial_MP = outcome(:,strmatch('obsS', M_.endo_names, 'exact'));

gdpGrowth_RANK = meanGDPGrowth + outcome(:,strmatch('obsgX_R', M_.endo_names, 'exact'));
cGrowth_RANK = meanCGrowth + outcome(:,strmatch('obsgC_R', M_.endo_names, 'exact'));
iGrowth_RANK = meanIGrowth + outcome(:,strmatch('obsgI_R', M_.endo_names, 'exact'));
gdpCycle_RANK = regcyc(cumsum(gdpGrowth_RANK)); 
gdpCycle_RANKPartial_MP = gdpCycle_RANK(12:end);
cCycle_RANK = regcyc(cumsum(cGrowth_RANK)); 
cCycle_RANKPartial_MP = cCycle_RANK(12:end);
iCycle_RANK = regcyc(cumsum(iGrowth_RANK)); 
iCycle_RANKPartial_MP = iCycle_RANK(12:end);
findingRate_RANKPartial_MP = outcome(:,strmatch('obsF_R', M_.endo_names, 'exact'));
lossRate_RANKPartial_MP = outcome(:,strmatch('obsS_R', M_.endo_names, 'exact'));


% Constant risk
dynare counterFactuals.mod -Dmodel_option=2 -DnMeasure_option=4

gdpGrowth_CRisk = meanGDPGrowth + outcome(:,strmatch('obsgX', M_.endo_names, 'exact'));
cGrowth_CRisk = meanCGrowth + outcome(:,strmatch('obsgC', M_.endo_names, 'exact'));
iGrowth_CRisk = meanIGrowth + outcome(:,strmatch('obsgI', M_.endo_names, 'exact'));
gdpCycle_CRisk = regcyc(cumsum(gdpGrowth_CRisk)); 
gdpCycle_CRiskPartial_MP = gdpCycle_CRisk(12:end);
cCycle_CRisk = regcyc(cumsum(cGrowth_CRisk)); 
cCycle_CRiskPartial_MP = cCycle_CRisk(12:end);
iCycle_CRisk = regcyc(cumsum(iGrowth_CRisk)); 
iCycle_CRiskPartial_MP = iCycle_CRisk(12:end);
findingRate_CRiskPartial_MP = outcome(:,strmatch('obsF', M_.endo_names, 'exact'));
lossRate_CRiskPartial_MP = outcome(:,strmatch('obsS', M_.endo_names, 'exact'));  


stdTablePartial_MP = [std(gdpCycle_HANKPartial_MP) std(cCycle_HANKPartial_MP) std(iCycle_HANKPartial_MP);
                  std(gdpCycle_CRiskPartial_MP) std(cCycle_CRiskPartial_MP) std(iCycle_CRiskPartial_MP);
                  std(gdpCycle_RANKPartial_MP) std(cCycle_RANKPartial_MP) std(iCycle_RANKPartial_MP)]; 

disp('Contribution of unemployment risk...')
disp((stdTablePartial_MP(1,2)-stdTablePartial_MP(2,2))/(stdTablePartial_MP(1,2)-stdTablePartial_MP(3,2)))
                            
              
%--- High adjustment cost ---%

load('HANKparam')
setParameters;
ss2prime = 20; 


save economicParameters.mat bbetaL ssigma aaBar bbetaH Hshare Hshare_T aalpha ddelta ss2prime rrhouk ppsi ...
     xxiP iiotaP ggamma rrhoxBar kkappa MMBar bb iiotaW xxiW FF rrhoR pphiPi pphiX vEpsilonGrid ...
     rrhoetaP rrhomuu rrhoupsilon rrhoTFP rrhozeta rrhorhox rrhopiStar rrhog... 
     ssigmaetaP ssigmamuu ssigmaupsilon ssigmaTFP ssigmaR ssigmazeta ssigmarhox...
     ssigmapiStar ssigmag ssigmaW;
 
 
shockNames = {'epsetaP', 'epsmuu', 'epsupsilon', 'epsTFP', 'epsR', 'epszeta', 'epsrhox', 'epsg', 'epsW'};

load('smoothedShocks')
load('firstSmoothedValues')

meanCGrowth = 0.53;
meanIGrowth = 0.45;
meanGDPGrowth = 0.47;

smoothedShocks(:,find(strcmp(shockNames, 'epszeta'))) = zeros(size(smoothedShocks,1),1);   
 

% HANK 
dynare counterFactuals.mod -Dmodel_option=1 -DnMeasure_option=4

gdpGrowth_HANK = meanGDPGrowth + outcome(:,strmatch('obsgX', M_.endo_names, 'exact'));
cGrowth_HANK = meanCGrowth + outcome(:,strmatch('obsgC', M_.endo_names, 'exact'));
iGrowth_HANK = meanIGrowth + outcome(:,strmatch('obsgI', M_.endo_names, 'exact'));
gdpCycle_HANK = regcyc(cumsum(gdpGrowth_HANK)); 
gdpCycle_HANKPartial_Iadj = gdpCycle_HANK(12:end);
cCycle_HANK = regcyc(cumsum(cGrowth_HANK)); 
cCycle_HANKPartial_Iadj = cCycle_HANK(12:end);
iCycle_HANK = regcyc(cumsum(iGrowth_HANK)); 
iCycle_HANKPartial_Iadj = iCycle_HANK(12:end);
findingRate_HANKPartial_Iadj = outcome(:,strmatch('obsF', M_.endo_names, 'exact'));
lossRate_HANKPartial_Iadj = outcome(:,strmatch('obsS', M_.endo_names, 'exact'));

gdpGrowth_RANK = meanGDPGrowth + outcome(:,strmatch('obsgX_R', M_.endo_names, 'exact'));
cGrowth_RANK = meanCGrowth + outcome(:,strmatch('obsgC_R', M_.endo_names, 'exact'));
iGrowth_RANK = meanIGrowth + outcome(:,strmatch('obsgI_R', M_.endo_names, 'exact'));
gdpCycle_RANK = regcyc(cumsum(gdpGrowth_RANK)); 
gdpCycle_RANKPartial_Iadj = gdpCycle_RANK(12:end);
cCycle_RANK = regcyc(cumsum(cGrowth_RANK)); 
cCycle_RANKPartial_Iadj = cCycle_RANK(12:end);
iCycle_RANK = regcyc(cumsum(iGrowth_RANK)); 
iCycle_RANKPartial_Iadj = iCycle_RANK(12:end);
findingRate_RANKPartial_Iadj = outcome(:,strmatch('obsF_R', M_.endo_names, 'exact'));
lossRate_RANKPartial_Iadj = outcome(:,strmatch('obsS_R', M_.endo_names, 'exact'));


% Constant risk
dynare counterFactuals.mod -Dmodel_option=2 -DnMeasure_option=4

gdpGrowth_CRisk = meanGDPGrowth + outcome(:,strmatch('obsgX', M_.endo_names, 'exact'));
cGrowth_CRisk = meanCGrowth + outcome(:,strmatch('obsgC', M_.endo_names, 'exact'));
iGrowth_CRisk = meanIGrowth + outcome(:,strmatch('obsgI', M_.endo_names, 'exact'));
gdpCycle_CRisk = regcyc(cumsum(gdpGrowth_CRisk)); 
gdpCycle_CRiskPartial_Iadj = gdpCycle_CRisk(12:end);
cCycle_CRisk = regcyc(cumsum(cGrowth_CRisk)); 
cCycle_CRiskPartial_Iadj = cCycle_CRisk(12:end);
iCycle_CRisk = regcyc(cumsum(iGrowth_CRisk)); 
iCycle_CRiskPartial_Iadj = iCycle_CRisk(12:end);
findingRate_CRiskPartial_Iadj = outcome(:,strmatch('obsF', M_.endo_names, 'exact'));
lossRate_CRiskPartial_Iadj = outcome(:,strmatch('obsS', M_.endo_names, 'exact'));  


stdTablePartial_Iadj = [std(gdpCycle_HANKPartial_Iadj) std(cCycle_HANKPartial_Iadj) std(iCycle_HANKPartial_Iadj);
                  std(gdpCycle_CRiskPartial_Iadj) std(cCycle_CRiskPartial_Iadj) std(iCycle_CRiskPartial_Iadj);
                  std(gdpCycle_RANKPartial_Iadj) std(cCycle_RANKPartial_Iadj) std(iCycle_RANKPartial_Iadj)]; 

disp('Contribution of unemployment risk...')
disp((stdTablePartial_Iadj(1,2)-stdTablePartial_Iadj(2,2))/(stdTablePartial_Iadj(1,2)-stdTablePartial_Iadj(3,2)))
              
%--- High wage rigidity ---%

load('HANKparam')
setParameters;
xxiW = 0.01;

save economicParameters.mat bbetaL ssigma aaBar bbetaH Hshare Hshare_T aalpha ddelta ss2prime rrhouk ppsi ...
     xxiP iiotaP ggamma rrhoxBar kkappa MMBar bb iiotaW xxiW FF rrhoR pphiPi pphiX vEpsilonGrid ...
     rrhoetaP rrhomuu rrhoupsilon rrhoTFP rrhozeta rrhorhox rrhopiStar rrhog... 
     ssigmaetaP ssigmamuu ssigmaupsilon ssigmaTFP ssigmaR ssigmazeta ssigmarhox...
     ssigmapiStar ssigmag ssigmaW;
 
shockNames = {'epsetaP', 'epsmuu', 'epsupsilon', 'epsTFP', 'epsR', 'epszeta', 'epsrhox', 'epsg', 'epsW'};

load('smoothedShocks')
load('firstSmoothedValues')

meanCGrowth = 0.53;
meanIGrowth = 0.45;
meanGDPGrowth = 0.47;

smoothedShocks(:,find(strcmp(shockNames, 'epszeta'))) = zeros(size(smoothedShocks,1),1);   
 

% HANK
dynare counterFactuals.mod -Dmodel_option=1 -DnMeasure_option=4

gdpGrowth_HANK = meanGDPGrowth + outcome(:,strmatch('obsgX', M_.endo_names, 'exact'));
cGrowth_HANK = meanCGrowth + outcome(:,strmatch('obsgC', M_.endo_names, 'exact'));
iGrowth_HANK = meanIGrowth + outcome(:,strmatch('obsgI', M_.endo_names, 'exact'));
gdpCycle_HANK = regcyc(cumsum(gdpGrowth_HANK)); 
gdpCycle_HANKPartial_Wage = gdpCycle_HANK(12:end);
cCycle_HANK = regcyc(cumsum(cGrowth_HANK)); 
cCycle_HANKPartial_Wage = cCycle_HANK(12:end);
iCycle_HANK = regcyc(cumsum(iGrowth_HANK)); 
iCycle_HANKPartial_Wage = iCycle_HANK(12:end);
findingRate_HANKPartial_Wage = outcome(:,strmatch('obsF', M_.endo_names, 'exact'));
lossRate_HANKPartial_Wage = outcome(:,strmatch('obsS', M_.endo_names, 'exact'));

gdpGrowth_RANK = meanGDPGrowth + outcome(:,strmatch('obsgX_R', M_.endo_names, 'exact'));
cGrowth_RANK = meanCGrowth + outcome(:,strmatch('obsgC_R', M_.endo_names, 'exact'));
iGrowth_RANK = meanIGrowth + outcome(:,strmatch('obsgI_R', M_.endo_names, 'exact'));
gdpCycle_RANK = regcyc(cumsum(gdpGrowth_RANK)); 
gdpCycle_RANKPartial_Wage = gdpCycle_RANK(12:end);
cCycle_RANK = regcyc(cumsum(cGrowth_RANK)); 
cCycle_RANKPartial_Wage = cCycle_RANK(12:end);
iCycle_RANK = regcyc(cumsum(iGrowth_RANK)); 
iCycle_RANKPartial_Wage = iCycle_RANK(12:end);
findingRate_RANKPartial_Wage = outcome(:,strmatch('obsF_R', M_.endo_names, 'exact'));
lossRate_RANKPartial_Wage = outcome(:,strmatch('obsS_R', M_.endo_names, 'exact'));


% Constant risk
dynare counterFactuals.mod -Dmodel_option=2 -DnMeasure_option=4

gdpGrowth_CRisk = meanGDPGrowth + outcome(:,strmatch('obsgX', M_.endo_names, 'exact'));
cGrowth_CRisk = meanCGrowth + outcome(:,strmatch('obsgC', M_.endo_names, 'exact'));
iGrowth_CRisk = meanIGrowth + outcome(:,strmatch('obsgI', M_.endo_names, 'exact'));
gdpCycle_CRisk = regcyc(cumsum(gdpGrowth_CRisk)); 
gdpCycle_CRiskPartial_Wage = gdpCycle_CRisk(12:end);
cCycle_CRisk = regcyc(cumsum(cGrowth_CRisk)); 
cCycle_CRiskPartial_Wage = cCycle_CRisk(12:end);
iCycle_CRisk = regcyc(cumsum(iGrowth_CRisk)); 
iCycle_CRiskPartial_Wage = iCycle_CRisk(12:end);
findingRate_CRiskPartial_Wage = outcome(:,strmatch('obsF', M_.endo_names, 'exact'));
lossRate_CRiskPartial_Wage = outcome(:,strmatch('obsS', M_.endo_names, 'exact'));  


stdTablePartial_Wage = [std(gdpCycle_HANKPartial_Wage) std(cCycle_HANKPartial_Wage) std(iCycle_HANKPartial_Wage);
                  std(gdpCycle_CRiskPartial_Wage) std(cCycle_CRiskPartial_Wage) std(iCycle_CRiskPartial_Wage);
                  std(gdpCycle_RANKPartial_Wage) std(cCycle_RANKPartial_Wage) std(iCycle_RANKPartial_Wage)]; 

disp('Contribution of unemployment risk...')
disp((stdTablePartial_Wage(1,2)-stdTablePartial_Wage(2,2))/(stdTablePartial_Wage(1,2)-stdTablePartial_Wage(3,2)))

              
%--- EU consumption differential of 30% ---%


calibrationType = 1;
steadyState_differentCalib;
 
shockNames = {'epsetaP', 'epsmuu', 'epsupsilon', 'epsTFP', 'epsR', 'epszeta', 'epsrhox', 'epsg', 'epsW'};

load('smoothedShocks')
load('firstSmoothedValues')

meanCGrowth = 0.53;
meanIGrowth = 0.45;
meanGDPGrowth = 0.47;

smoothedShocks(:,find(strcmp(shockNames, 'epszeta'))) = zeros(size(smoothedShocks,1),1);   
 

% HANK
dynare counterFactuals.mod -Dmodel_option=1 -DnMeasure_option=4

gdpGrowth_HANK = meanGDPGrowth + outcome(:,strmatch('obsgX', M_.endo_names, 'exact'));
cGrowth_HANK = meanCGrowth + outcome(:,strmatch('obsgC', M_.endo_names, 'exact'));
iGrowth_HANK = meanIGrowth + outcome(:,strmatch('obsgI', M_.endo_names, 'exact'));
gdpCycle_HANK = regcyc(cumsum(gdpGrowth_HANK)); 
gdpCycle_HANKPartial_highEUC = gdpCycle_HANK(12:end);
cCycle_HANK = regcyc(cumsum(cGrowth_HANK)); 
cCycle_HANKPartial_highEUC = cCycle_HANK(12:end);
iCycle_HANK = regcyc(cumsum(iGrowth_HANK)); 
iCycle_HANKPartial_highEUC = iCycle_HANK(12:end);
findingRate_HANKPartial_highEUC = outcome(:,strmatch('obsF', M_.endo_names, 'exact'));
lossRate_HANKPartial_highEUC = outcome(:,strmatch('obsS', M_.endo_names, 'exact'));

gdpGrowth_RANK = meanGDPGrowth + outcome(:,strmatch('obsgX_R', M_.endo_names, 'exact'));
cGrowth_RANK = meanCGrowth + outcome(:,strmatch('obsgC_R', M_.endo_names, 'exact'));
iGrowth_RANK = meanIGrowth + outcome(:,strmatch('obsgI_R', M_.endo_names, 'exact'));
gdpCycle_RANK = regcyc(cumsum(gdpGrowth_RANK)); 
gdpCycle_RANKPartial_highEUC = gdpCycle_RANK(12:end);
cCycle_RANK = regcyc(cumsum(cGrowth_RANK)); 
cCycle_RANKPartial_highEUC = cCycle_RANK(12:end);
iCycle_RANK = regcyc(cumsum(iGrowth_RANK)); 
iCycle_RANKPartial_highEUC = iCycle_RANK(12:end);
findingRate_RANKPartial_highEUC = outcome(:,strmatch('obsF_R', M_.endo_names, 'exact'));
lossRate_RANKPartial_highEUC = outcome(:,strmatch('obsS_R', M_.endo_names, 'exact'));


% Constant risk
dynare counterFactuals.mod -Dmodel_option=2 -DnMeasure_option=4

gdpGrowth_CRisk = meanGDPGrowth + outcome(:,strmatch('obsgX', M_.endo_names, 'exact'));
cGrowth_CRisk = meanCGrowth + outcome(:,strmatch('obsgC', M_.endo_names, 'exact'));
iGrowth_CRisk = meanIGrowth + outcome(:,strmatch('obsgI', M_.endo_names, 'exact'));
gdpCycle_CRisk = regcyc(cumsum(gdpGrowth_CRisk)); 
gdpCycle_CRiskPartial_highEUC = gdpCycle_CRisk(12:end);
cCycle_CRisk = regcyc(cumsum(cGrowth_CRisk)); 
cCycle_CRiskPartial_highEUC = cCycle_CRisk(12:end);
iCycle_CRisk = regcyc(cumsum(iGrowth_CRisk)); 
iCycle_CRiskPartial_highEUC = iCycle_CRisk(12:end);
findingRate_CRiskPartial_highEUC = outcome(:,strmatch('obsF', M_.endo_names, 'exact'));
lossRate_CRiskPartial_highEUC = outcome(:,strmatch('obsS', M_.endo_names, 'exact'));  


stdTablePartial_highEUC = [std(gdpCycle_HANKPartial_highEUC) std(cCycle_HANKPartial_highEUC) std(iCycle_HANKPartial_highEUC);
                  std(gdpCycle_CRiskPartial_highEUC) std(cCycle_CRiskPartial_highEUC) std(iCycle_CRiskPartial_highEUC);
                  std(gdpCycle_RANKPartial_highEUC) std(cCycle_RANKPartial_highEUC) std(iCycle_RANKPartial_highEUC)]; 

disp('Contribution of unemployment risk...')
disp((stdTablePartial_highEUC(1,2)-stdTablePartial_highEUC(2,2))/(stdTablePartial_highEUC(1,2)-stdTablePartial_highEUC(3,2)))
              
%--- EU consumption differential of 10% ---%
calibrationType = 2;
steadyState_differentCalib;
 
shockNames = {'epsetaP', 'epsmuu', 'epsupsilon', 'epsTFP', 'epsR', 'epszeta', 'epsrhox', 'epsg', 'epsW'};

load('smoothedShocks')
load('firstSmoothedValues')

meanCGrowth = 0.53;
meanIGrowth = 0.45;
meanGDPGrowth = 0.47;

smoothedShocks(:,find(strcmp(shockNames, 'epszeta'))) = zeros(size(smoothedShocks,1),1);   
 

% HANK 
dynare counterFactuals.mod -Dmodel_option=1 -DnMeasure_option=4

gdpGrowth_HANK = meanGDPGrowth + outcome(:,strmatch('obsgX', M_.endo_names, 'exact'));
cGrowth_HANK = meanCGrowth + outcome(:,strmatch('obsgC', M_.endo_names, 'exact'));
iGrowth_HANK = meanIGrowth + outcome(:,strmatch('obsgI', M_.endo_names, 'exact'));
gdpCycle_HANK = regcyc(cumsum(gdpGrowth_HANK)); 
gdpCycle_HANKPartial_lowEUC = gdpCycle_HANK(12:end);
cCycle_HANK = regcyc(cumsum(cGrowth_HANK)); 
cCycle_HANKPartial_lowEUC = cCycle_HANK(12:end);
iCycle_HANK = regcyc(cumsum(iGrowth_HANK)); 
iCycle_HANKPartial_lowEUC = iCycle_HANK(12:end);
findingRate_HANKPartial_lowEUC = outcome(:,strmatch('obsF', M_.endo_names, 'exact'));
lossRate_HANKPartial_lowEUC = outcome(:,strmatch('obsS', M_.endo_names, 'exact'));

gdpGrowth_RANK = meanGDPGrowth + outcome(:,strmatch('obsgX_R', M_.endo_names, 'exact'));
cGrowth_RANK = meanCGrowth + outcome(:,strmatch('obsgC_R', M_.endo_names, 'exact'));
iGrowth_RANK = meanIGrowth + outcome(:,strmatch('obsgI_R', M_.endo_names, 'exact'));
gdpCycle_RANK = regcyc(cumsum(gdpGrowth_RANK)); 
gdpCycle_RANKPartial_lowEUC = gdpCycle_RANK(12:end);
cCycle_RANK = regcyc(cumsum(cGrowth_RANK)); 
cCycle_RANKPartial_lowEUC = cCycle_RANK(12:end);
iCycle_RANK = regcyc(cumsum(iGrowth_RANK)); 
iCycle_RANKPartial_lowEUC = iCycle_RANK(12:end);
findingRate_RANKPartial_lowEUC = outcome(:,strmatch('obsF_R', M_.endo_names, 'exact'));
lossRate_RANKPartial_lowEUC = outcome(:,strmatch('obsS_R', M_.endo_names, 'exact'));


% Constant risk
dynare counterFactuals.mod -Dmodel_option=2 -DnMeasure_option=4

gdpGrowth_CRisk = meanGDPGrowth + outcome(:,strmatch('obsgX', M_.endo_names, 'exact'));
cGrowth_CRisk = meanCGrowth + outcome(:,strmatch('obsgC', M_.endo_names, 'exact'));
iGrowth_CRisk = meanIGrowth + outcome(:,strmatch('obsgI', M_.endo_names, 'exact'));
gdpCycle_CRisk = regcyc(cumsum(gdpGrowth_CRisk)); 
gdpCycle_CRiskPartial_lowEUC = gdpCycle_CRisk(12:end);
cCycle_CRisk = regcyc(cumsum(cGrowth_CRisk)); 
cCycle_CRiskPartial_lowEUC = cCycle_CRisk(12:end);
iCycle_CRisk = regcyc(cumsum(iGrowth_CRisk)); 
iCycle_CRiskPartial_lowEUC = iCycle_CRisk(12:end);
findingRate_CRiskPartial_lowEUC = outcome(:,strmatch('obsF', M_.endo_names, 'exact'));
lossRate_CRiskPartial_lowEUC = outcome(:,strmatch('obsS', M_.endo_names, 'exact'));  


stdTablePartial_lowEUC = [std(gdpCycle_HANKPartial_lowEUC) std(cCycle_HANKPartial_lowEUC) std(iCycle_HANKPartial_lowEUC);
                  std(gdpCycle_CRiskPartial_lowEUC) std(cCycle_CRiskPartial_lowEUC) std(iCycle_CRiskPartial_lowEUC);
                  std(gdpCycle_RANKPartial_lowEUC) std(cCycle_RANKPartial_lowEUC) std(iCycle_RANKPartial_lowEUC)]; 

disp('Contribution of unemployment risk...')
disp((stdTablePartial_lowEUC(1,2)-stdTablePartial_lowEUC(2,2))/(stdTablePartial_lowEUC(1,2)-stdTablePartial_lowEUC(3,2)))              
              
%--- MPC = 0.1 ---%
calibrationType = 3; MPCType = 2;
steadyState_differentCalib;
 
shockNames = {'epsetaP', 'epsmuu', 'epsupsilon', 'epsTFP', 'epsR', 'epszeta', 'epsrhox', 'epsg', 'epsW'};

load('smoothedShocks')
load('firstSmoothedValues')

meanCGrowth = 0.53;
meanIGrowth = 0.45;
meanGDPGrowth = 0.47;

smoothedShocks(:,find(strcmp(shockNames, 'epszeta'))) = zeros(size(smoothedShocks,1),1);   

dynare counterFactuals.mod -Dmodel_option=1 -DnMeasure_option=4

load('grids');
vAssetsGridFine_lowMPC = vAssetsGridFine; 
mDistributionFine_lowMPC = mDistributionFine;


gdpGrowth_HANK = meanGDPGrowth + outcome(:,strmatch('obsgX', M_.endo_names, 'exact'));
cGrowth_HANK = meanCGrowth + outcome(:,strmatch('obsgC', M_.endo_names, 'exact'));
iGrowth_HANK = meanIGrowth + outcome(:,strmatch('obsgI', M_.endo_names, 'exact'));
gdpCycle_HANK = regcyc(cumsum(gdpGrowth_HANK)); 
gdpCycle_HANKPartial_lowMPC = gdpCycle_HANK(12:end);
cCycle_HANK = regcyc(cumsum(cGrowth_HANK)); 
cCycle_HANKPartial_lowMPC = cCycle_HANK(12:end);
iCycle_HANK = regcyc(cumsum(iGrowth_HANK)); 
iCycle_HANKPartial_lowMPC = iCycle_HANK(12:end);
findingRate_HANKPartial_lowMPC = outcome(:,strmatch('obsF', M_.endo_names, 'exact'));
lossRate_HANKPartial_lowMPC = outcome(:,strmatch('obsS', M_.endo_names, 'exact'));

gdpGrowth_RANK = meanGDPGrowth + outcome(:,strmatch('obsgX_R', M_.endo_names, 'exact'));
cGrowth_RANK = meanCGrowth + outcome(:,strmatch('obsgC_R', M_.endo_names, 'exact'));
iGrowth_RANK = meanIGrowth + outcome(:,strmatch('obsgI_R', M_.endo_names, 'exact'));
gdpCycle_RANK = regcyc(cumsum(gdpGrowth_RANK)); 
gdpCycle_RANKPartial_lowMPC = gdpCycle_RANK(12:end);
cCycle_RANK = regcyc(cumsum(cGrowth_RANK)); 
cCycle_RANKPartial_lowMPC = cCycle_RANK(12:end);
iCycle_RANK = regcyc(cumsum(iGrowth_RANK)); 
iCycle_RANKPartial_lowMPC = iCycle_RANK(12:end);
findingRate_RANKPartial_lowMPC = outcome(:,strmatch('obsF_R', M_.endo_names, 'exact'));
lossRate_RANKPartial_lowMPC = outcome(:,strmatch('obsS_R', M_.endo_names, 'exact'));


% Constant risk
dynare counterFactuals.mod -Dmodel_option=2 -DnMeasure_option=4

gdpGrowth_CRisk = meanGDPGrowth + outcome(:,strmatch('obsgX', M_.endo_names, 'exact'));
cGrowth_CRisk = meanCGrowth + outcome(:,strmatch('obsgC', M_.endo_names, 'exact'));
iGrowth_CRisk = meanIGrowth + outcome(:,strmatch('obsgI', M_.endo_names, 'exact'));
gdpCycle_CRisk = regcyc(cumsum(gdpGrowth_CRisk)); 
gdpCycle_CRiskPartial_lowMPC = gdpCycle_CRisk(12:end);
cCycle_CRisk = regcyc(cumsum(cGrowth_CRisk)); 
cCycle_CRiskPartial_lowMPC = cCycle_CRisk(12:end);
iCycle_CRisk = regcyc(cumsum(iGrowth_CRisk)); 
iCycle_CRiskPartial_lowMPC = iCycle_CRisk(12:end);
findingRate_CRiskPartial_lowMPC = outcome(:,strmatch('obsF', M_.endo_names, 'exact'));
lossRate_CRiskPartial_lowMPC = outcome(:,strmatch('obsS', M_.endo_names, 'exact'));  


stdTablePartial_lowMPC = [std(gdpCycle_HANKPartial_lowMPC) std(cCycle_HANKPartial_lowMPC) std(iCycle_HANKPartial_lowMPC);
                  std(gdpCycle_CRiskPartial_lowMPC) std(cCycle_CRiskPartial_lowMPC) std(iCycle_CRiskPartial_lowMPC);
                  std(gdpCycle_RANKPartial_lowMPC) std(cCycle_RANKPartial_lowMPC) std(iCycle_RANKPartial_lowMPC)]; 

disp('Contribution of unemployment risk...')
disp((stdTablePartial_lowMPC(1,2)-stdTablePartial_lowMPC(2,2))/(stdTablePartial_lowMPC(1,2)-stdTablePartial_lowMPC(3,2)))  


%--- Liquid wealth to GDP Ratio = 0.35 ---%

calibrationType = 4;
steadyState_differentCalib;
 
shockNames = {'epsetaP', 'epsmuu', 'epsupsilon', 'epsTFP', 'epsR', 'epszeta', 'epsrhox', 'epsg', 'epsW'};

load('smoothedShocks')
load('firstSmoothedValues')

meanCGrowth = 0.53;
meanIGrowth = 0.45;
meanGDPGrowth = 0.47;

smoothedShocks(:,find(strcmp(shockNames, 'epszeta'))) = zeros(size(smoothedShocks,1),1);   


% HANK
dynare counterFactuals.mod -Dmodel_option=1 -DnMeasure_option=4

gdpGrowth_HANK = meanGDPGrowth + outcome(:,strmatch('obsgX', M_.endo_names, 'exact'));
cGrowth_HANK = meanCGrowth + outcome(:,strmatch('obsgC', M_.endo_names, 'exact'));
iGrowth_HANK = meanIGrowth + outcome(:,strmatch('obsgI', M_.endo_names, 'exact'));
gdpCycle_HANK = regcyc(cumsum(gdpGrowth_HANK)); 
gdpCycle_HANKPartial_highWealth = gdpCycle_HANK(12:end);
cCycle_HANK = regcyc(cumsum(cGrowth_HANK)); 
cCycle_HANKPartial_highWealth = cCycle_HANK(12:end);
iCycle_HANK = regcyc(cumsum(iGrowth_HANK)); 
iCycle_HANKPartial_highWealth = iCycle_HANK(12:end);
findingRate_HANKPartial_highWealth = outcome(:,strmatch('obsF', M_.endo_names, 'exact'));
lossRate_HANKPartial_highWealth = outcome(:,strmatch('obsS', M_.endo_names, 'exact'));

gdpGrowth_RANK = meanGDPGrowth + outcome(:,strmatch('obsgX_R', M_.endo_names, 'exact'));
cGrowth_RANK = meanCGrowth + outcome(:,strmatch('obsgC_R', M_.endo_names, 'exact'));
iGrowth_RANK = meanIGrowth + outcome(:,strmatch('obsgI_R', M_.endo_names, 'exact'));
gdpCycle_RANK = regcyc(cumsum(gdpGrowth_RANK)); 
gdpCycle_RANKPartial_highWealth = gdpCycle_RANK(12:end);
cCycle_RANK = regcyc(cumsum(cGrowth_RANK)); 
cCycle_RANKPartial_highWealth = cCycle_RANK(12:end);
iCycle_RANK = regcyc(cumsum(iGrowth_RANK)); 
iCycle_RANKPartial_highWealth = iCycle_RANK(12:end);
findingRate_RANKPartial_highWealth = outcome(:,strmatch('obsF_R', M_.endo_names, 'exact'));
lossRate_RANKPartial_highWealth = outcome(:,strmatch('obsS_R', M_.endo_names, 'exact'));


% Constant risk
dynare counterFactuals.mod -Dmodel_option=2 -DnMeasure_option=4

gdpGrowth_CRisk = meanGDPGrowth + outcome(:,strmatch('obsgX', M_.endo_names, 'exact'));
cGrowth_CRisk = meanCGrowth + outcome(:,strmatch('obsgC', M_.endo_names, 'exact'));
iGrowth_CRisk = meanIGrowth + outcome(:,strmatch('obsgI', M_.endo_names, 'exact'));
gdpCycle_CRisk = regcyc(cumsum(gdpGrowth_CRisk)); 
gdpCycle_CRiskPartial_highWealth = gdpCycle_CRisk(12:end);
cCycle_CRisk = regcyc(cumsum(cGrowth_CRisk)); 
cCycle_CRiskPartial_highWealth = cCycle_CRisk(12:end);
iCycle_CRisk = regcyc(cumsum(iGrowth_CRisk)); 
iCycle_CRiskPartial_highWealth = iCycle_CRisk(12:end);
findingRate_CRiskPartial_highWealth = outcome(:,strmatch('obsF', M_.endo_names, 'exact'));
lossRate_CRiskPartial_highWealth = outcome(:,strmatch('obsS', M_.endo_names, 'exact'));  


stdTablePartial_highWealth = [std(gdpCycle_HANKPartial_highWealth) std(cCycle_HANKPartial_highWealth) std(iCycle_HANKPartial_highWealth);
                  std(gdpCycle_CRiskPartial_highWealth) std(cCycle_CRiskPartial_highWealth) std(iCycle_CRiskPartial_highWealth);
                  std(gdpCycle_RANKPartial_highWealth) std(cCycle_RANKPartial_highWealth) std(iCycle_RANKPartial_highWealth)]; 

disp('Contribution of unemployment risk...')
disp((stdTablePartial_highWealth(1,2)-stdTablePartial_highWealth(2,2))/(stdTablePartial_highWealth(1,2)-stdTablePartial_highWealth(3,2)))  


%------------------------------------------------------------------------
% Compute moments of aggregate variables subject to all shocks
% under different degree of approximation to the distribution
% Produces Table A.1
%------------------------------------------------------------------------

%nMeasure = 2
calibrationType = 5; nMeasureType = 2;
steadyState_differentCalib;

dynare HANKDynamicsShocks.mod -Dmodel_option=1 -DnMeasure_option=2

x_pos = strmatch('obsgX', M_.endo_names, 'exact');
c_pos = strmatch('obsgC', M_.endo_names, 'exact');
i_pos = strmatch('obsgI', M_.endo_names, 'exact');
pi_pos = strmatch('obsPi', M_.endo_names, 'exact');
f_pos = strmatch('obsF', M_.endo_names, 'exact');
s_pos = strmatch('obsS', M_.endo_names, 'exact');
u_pos = strmatch('obsU', M_.endo_names, 'exact');
v_pos = strmatch('vHat', M_.endo_names, 'exact');


stdTable_HANKmodel_nMeasure2 = [sqrt(oo_.var(c_pos,c_pos)) sqrt(oo_.var(i_pos,i_pos)) sqrt(oo_.var(pi_pos,pi_pos))...
                    sqrt(oo_.var(f_pos,f_pos)) sqrt(oo_.var(s_pos,s_pos))];   
                     
%nMeasure = 3
calibrationType = 5; nMeasureType = 3;
steadyState_differentCalib;

dynare HANKDynamicsShocks.mod -Dmodel_option=1 -DnMeasure_option=3

x_pos = strmatch('obsgX', M_.endo_names, 'exact');
c_pos = strmatch('obsgC', M_.endo_names, 'exact');
i_pos = strmatch('obsgI', M_.endo_names, 'exact');
pi_pos = strmatch('obsPi', M_.endo_names, 'exact');
f_pos = strmatch('obsF', M_.endo_names, 'exact');
s_pos = strmatch('obsS', M_.endo_names, 'exact');
u_pos = strmatch('obsU', M_.endo_names, 'exact');
v_pos = strmatch('vHat', M_.endo_names, 'exact');


stdTable_HANKmodel_nMeasure3 = [sqrt(oo_.var(c_pos,c_pos)) sqrt(oo_.var(i_pos,i_pos)) sqrt(oo_.var(pi_pos,pi_pos))...
                    sqrt(oo_.var(f_pos,f_pos)) sqrt(oo_.var(s_pos,s_pos))]; 

%------------------------------------------------------------------------
% Stationary wealth distribution and decision rules:
% (smoothed) histogram vs. parametric function
% Produces Figure A.1
%------------------------------------------------------------------------
calibrationType = 5; nMeasureType = 4;
steadyState_differentCalib;

dynare HANKDynamicsShocks.mod -Dmodel_option=1 -DnMeasure_option=4

x_pos = strmatch('obsgX', M_.endo_names, 'exact');
c_pos = strmatch('obsgC', M_.endo_names, 'exact');
i_pos = strmatch('obsgI', M_.endo_names, 'exact');
pi_pos = strmatch('obsPi', M_.endo_names, 'exact');
f_pos = strmatch('obsF', M_.endo_names, 'exact');
s_pos = strmatch('obsS', M_.endo_names, 'exact');
u_pos = strmatch('obsU', M_.endo_names, 'exact');
v_pos = strmatch('vHat', M_.endo_names, 'exact');


stdTable_HANKmodel_nMeasure4 = [sqrt(oo_.var(c_pos,c_pos)) sqrt(oo_.var(i_pos,i_pos)) sqrt(oo_.var(pi_pos,pi_pos))...
                    sqrt(oo_.var(f_pos,f_pos)) sqrt(oo_.var(s_pos,s_pos))]; 

                
stdTable_HANKmodel_nMeasure = [stdTable_HANKmodel_nMeasure2;stdTable_HANKmodel_nMeasure3;stdTable_HANKmodel_nMeasure4];

%nMeasure = 3
calibrationType = 5; nMeasureType = 4;
steadyState_differentCalib;

mDistributionFine_nMeasure4 = mDistributionFine;


load('grids');
load('steadyStateAggregates')

% Distribution 
FigHandle = figure('Position', [150, 150, 1600, 600]);
set(0,'defaultlinelinewidth',2.5,'defaultLineMarkerSize',8);

subaxis(1,2,1,'spacing',0.1);
hold on
smoothDist_unemp = smoothdata(mHistogram_hist(1,:),'movmean',7)/ sum(smoothdata(mHistogram_hist(1,:),'movmean',7));
plot(vAssetsGridFine/ySS, [mHatHistogram(1) smoothDist_unemp(2:end)]/sum([mHatHistogram(1) smoothDist_unemp(2:end)]),... 
	'linewidth',1.5,'color',[8/255,62/255,118/255])
plot(vAssetsGridFine/ySS,[mHat(1) mDistributionFine_nMeasure4(1,2:end) ./ sum(mDistributionFine_nMeasure4(1,2:end))]/sum([mHat(1) mDistributionFine_nMeasure4(1,2:end) ./ sum(mDistributionFine_nMeasure4(1,2:end))]),...
	'linewidth',1.5,'color',[178/255,34/255,34/255],'linestyle','--')
hold off
xlabel('Liquid assets to output ratio','interpreter','latex')
ylabel({'Mass of households'},'Interpreter','latex')
grid on
xlim([0 0.2])
title({'Distribution (unemployed)'},'Interpreter','latex')
set(gca,'FontSize',15)
set(gcf,'color','w')
lg = legend({'Histogram','Parametric function ($n_g$ = 4)'},'Interpreter','latex','location','best');
legend('boxoff');

subaxis(1,2,2);
hold on
smoothDist_emp = smoothdata(mHistogram_hist(2,:),'movmean',7)/ sum(smoothdata(mHistogram_hist(2,:),'movmean',7));
plot(vAssetsGridFine/ySS,[mHatHistogram(2) smoothDist_emp(2:end)]/sum([mHatHistogram(2) smoothDist_emp(2:end)]),... 
	'linewidth',1.5,'color',[8/255,62/255,118/255])
plot(vAssetsGridFine/ySS,[mHat(2) mDistributionFine_nMeasure4(2,2:end) ./ sum(mDistributionFine_nMeasure4(1,2:end))]/sum([mHat(2) mDistributionFine_nMeasure4(2,2:end) ./ sum(mDistributionFine_nMeasure4(1,2:end))]),... 
	'linewidth',1.5,'color',[178/255,34/255,34/255],'linestyle','--')
hold off
xlabel('Liquid assets to output ratio','interpreter','latex')
ylabel({'Mass of households'},'Interpreter','latex')
grid on
xlim([0 0.2])
title({'Distribution (employed)'},'Interpreter','latex')
set(gca,'FontSize',15)
set(gcf,'color','w')

% Decision rules
FigHandle = figure('Position', [150, 150, 1600, 600]);
set(0,'defaultlinelinewidth',2.5,'defaultLineMarkerSize',8);

subaxis(1,2,1,'spacing',0.1)
hold on
plot(vAssetsGridFine/ySS,mConsumption_hist(1,:),'-','color',[8/255,62/255,118/255])
plot(vAssetsGridFine/ySS,mConsumption(1,:),'--','color',[178/255,34/255,34/255])
plot(vAssetsGridFine/ySS,vAssetsGridFine,'k--','linewidth',.5)
xlabel({'Liquid assets to output ratio'},'Interpreter','latex')
ylabel({'Consumption'},'Interpreter','latex')
xlim([0 0.6])
title({'Decision rule (Unemployed)'},'Interpreter','latex')
set(gca,'FontSize',15)
grid on
set(gcf,'color','w')
lg = legend({'Histogram','Parametric function ($n_g$ = 4)'},'Interpreter','latex','location','best');
set(lg,'FontSize',15);
legend('boxoff');
hold off

subaxis(1,2,2);
hold on
plot(vAssetsGridFine/ySS,mConsumption_hist(2,:),'-','color',[8/255,62/255,118/255])
plot(vAssetsGridFine/ySS,mConsumption(2,:),'--','color',[178/255,34/255,34/255])
plot(vAssetsGridFine/ySS,vAssetsGridFine,'k--','linewidth',.5)
hold off
xlabel({'Liquid assets to output ratio'},'Interpreter','latex')
ylabel({'Consumption'},'Interpreter','latex')
xlim([0 0.6])
title({'Decision rule (Employed)'},'Interpreter','latex')
set(gca,'FontSize',15)
grid on
set(gcf,'color','w')
hold off


delete HANKDynamicsShocks.m
rmdir HANKDynamicsShocks s
delete counterFactuals.m
rmdir counterFactuals s
delete RANKDynamics.m
rmdir RANKDynamics s



delete *_dynamic.m
delete *_static.m
delete *.log
delete *_set_auxiliary_variables.m
delete *_results.mat
delete *.eps
delete *.asv   


cd('../')